import CodeMirror from 'codemirror/lib/codemirror'
import highlightjs from 'highlightjs'

class CodeEditor {
  constructor(tinymceEditor, textarea) {
    const settings = tinymceEditor.settings.codeblock
    const theme = settings.codeTheme || 'default'

    this.tinymceEditor = tinymceEditor
    this.textarea = textarea
    if (CodeMirror && CodeMirror.fromTextArea) {
      this.codeMirror = CodeMirror.fromTextArea(textarea, {
        lineNumbers: true,
        autofocus: true,
        theme: theme,
      })
    }
  }

  refresh() {
    this.selectedNode = null
    if (this.codeMirror) {
      this.codeMirror.refresh()
      this.codeMirror.setValue(this.getCurrentCode())
      this.codeMirror.focus()
    } else {
      this.textarea.value = this.getCurrentCode()
      this.textarea.focus()
    }
  }

  insertCodeBlock(language) {
    const editor = this.tinymceEditor
    let code = this.getValue()
    let resultExc = false

    editor.undoManager.transact(() => {
      let node = this.getSelectedCodeBlock()
      code = editor.dom.encode(code)

      if (code.length == 0) {
        resultExc = false
      } else {
        if (node) {
          node.innerHTML = code
          editor.selection.select(node)
          node.className = language

          editor.undoManager.transact(() => {
            node.querySelectorAll('br').forEach(br => {
              br.parentNode.replaceChild(editor.getDoc().createTextNode('\n'), br)
            })

            node.contentEditable = false
            node.innerHTML = editor.dom.encode(node.textContent)
            if (highlightjs && highlightjs.highlightBlock) {
              highlightjs.highlightBlock(node)
            }
          })
        } else {
          const id = `code_${Date.now()}`
          editor.insertContent(`<pre id="${id}" class="${language}" data-ke-type="codeblock">${code}</pre>`)
          editor.selection.select(editor.$(id)[0])
        }
        resultExc = true
      }
    })
    return resultExc
  }

  getValue() {
    return this.codeMirror ? this.codeMirror.getValue() : this.textarea.value
  }

  getCurrentCode() {
    let node = this.getSelectedCodeBlock()
    if (node) {
      return node.textContent
    }

    return this.tinymceEditor.selection.getContent({ format: 'text' })
  }

  getSelectedCodeBlock() {
    if (!this.selectedNode) {
      let node = this.tinymceEditor.selection.getNode()
      if (node && node.nodeName == 'PRE') {
        this.selectedNode = node
      }
    }

    return this.selectedNode
  }

  onChange(callback) {
    if (this.codeMirror) {
      this.codeMirror.on('change', doc => {
        callback(doc.getValue())
      })
    } else {
      this.textarea.addEventListener('keyup', e => {
        callback(e.target.value)
      })
    }
  }
}

export default CodeEditor
