class Panel {
  editor: any
  panel: any
  modules: String[]
  defaultColor: any
  presetMap: any
  format: any
  blackTextColorBackgroundColor: String[]
  constructor(editor: any) {
    this.editor = editor
    const settings = editor.settings.colorPalette
    this.modules = (settings && settings.module) || ['preset']
    this.defaultColor = settings && settings.default_color
    this.blackTextColorBackgroundColor = settings && settings.black_text_color_background
    this.presetMap = settings && settings.color_map_preset
  }

  init(panel: any) {
    this.panel = panel
    this.format = panel.parent().settings.format
    this.addEvent()
  }

  addEvent() {
    const $ = this.editor.$
    const panelElem = this.panel.$el[0]
    const parentControl = this.panel.parent()

    const prefix = this.format === 'hilitecolor' ? 'hilite-' : 'forecolor-'
    if (this.modules.includes('preset')) {
      $(`.${prefix}colorPalette-preset-td`, panelElem).on('click', this.clickEventHandler.bind(this))
    }

    if (this.modules.includes('palette')) {
      $(`.${prefix}colorPalette-palette-td`, panelElem).on('click', this.clickEventHandler.bind(this))
    }

    if (this.modules.includes('code')) {
      const $input = $('.colorPalette-code-input', panelElem)
      const input = $input[0]
      const $displayBox = $('.colorPalette-code-display', panelElem)
      const $submitBtn = $('.colorPalette-code-submit', panelElem)

      const applyCodeColor = (codeValue: String) => {
        codeValue = codeValue.trim()
        if (codeValue.charAt(0) === '#' && (codeValue.length == 7 || codeValue.length == 4)) {
          if (this.isValidHexCode(codeValue)) {
            this.applyFormat(this.format, codeValue)
            parentControl.hidePanel()
          }
        }
      }

      $input.on('keyup', e => {
        if (e.keyCode === 13) {
          applyCodeColor(input.value)
        }
      })

      $submitBtn.on('click', () => {
        applyCodeColor(input.value)
      })
    }
  }

  applyFormat(format, value) {
    this.editor.execCommand('mceApplyTextcolor', format, value)
    if (format === 'forecolor') {
      this.panel.parent().color(value)
    } else if (format === 'hilitecolor') {
      this.panel.parent().color(value)
      const parentButton = this.panel.parent()
      const $iconElem = this.editor.$('i', parentButton.$el[0])
      if (this.blackTextColorBackgroundColor.includes(value)) {
        $iconElem.removeClass('white-text')
      } else {
        $iconElem.addClass('white-text')
      }
    }
    this.editor.nodeChanged()
  }

  removeFormat(format) {
    this.editor.execCommand('mceRemoveTextcolor', format)
    if (format === 'forecolor') {
      this.panel.parent().color(this.defaultColor)
    } else if (format === 'hilitecolor') {
      this.panel.parent().color('#fff')
      const parentButton = this.panel.parent()
      const $iconElem = this.editor.$('i', parentButton.$el[0])
      $iconElem.removeClass('white-text')
    }
    this.editor.nodeChanged()
  }

  clickEventHandler(e) {
    this.hide()
    const color = e.currentTarget.getAttribute('data-mce-color')
    if (color === '') {
      this.editor.undoManager.transact(() => {
        this.removeFormat(this.format)
      })
    } else {
      this.editor.undoManager.transact(() => {
        this.applyFormat(this.format, color)
      })
    }
  }

  convertToHexCode(colorString) {
    if (!colorString) {
      return this.format === 'forecolor' ? this.defaultColor : ''
    }

    const m = /rgb\((\d*), (\d*), (\d*)\)/gi.exec(colorString)

    return (
      '#' +
      m
        .slice(1)
        .reduce((res, cur) => {
          let parsedHex = parseInt(cur).toString(16)
          if (parsedHex.length % 2) {
            parsedHex = '0' + parsedHex
          }
          return res + parsedHex
        }, '')
        .toLowerCase()
    )
  }

  isValidHexCode(code) {
    const len = code.length
    if (len === 4 || len === 7) {
      if (code[0] !== '#') {
        return false
      }
      for (let i = 1; i < len; i++) {
        let charAtCode = code[i].toLowerCase()
        if (!((0 <= charAtCode && charAtCode <= 9) || ('a' <= charAtCode && charAtCode <= 'f'))) {
          return false
        }
      }
      return true
    }
    return false
  }

  getSpanColor() {
    const selectedNode = this.editor.selection.getSel().anchorNode
    const parentSpans = this.editor.dom.getParents(selectedNode, 'span')
    let parentSpan: { style: { color: String; backgroundColor: String } }
    for (let span of parentSpans) {
      if (this.format === 'forecolor' && span.id !== '_mce_caret' && span.style && span.style.color) {
        parentSpan = span
        break
      } else if (
        this.format === 'hilitecolor' &&
        span.id !== '_mce_caret' &&
        span.style &&
        span.style.backgroundColor
      ) {
        parentSpan = span
        break
      }
    }
    let color: String = this.format === 'forecolor' ? this.defaultColor : '#ffffff'
    if (parentSpan) {
      color = this.format === 'forecolor' ? parentSpan.style.color : parentSpan.style.backgroundColor
      color = this.convertToHexCode(color)
    }
    return color
  }

  show() {
    const color = this.getSpanColor()
    this.showSelectedColor(color)
    this.showCustomColor(color)
  }

  private showSelectedColor(color) {
    const selectedTd = this.editor.$.find(`#colorPalette-${this.format}-preset-id-${this.presetMap.indexOf(color)}`)[0]
    selectedTd && selectedTd.classList.add('colorPalette-selected')
  }

  private showCustomColor(color) {
    const panelElem = this.panel.$el[0]
    const previewElem = this.editor.$('.colorPalette-code-display', panelElem)[0]
    const inputElem = this.editor.$('.colorPalette-code-input', panelElem)[0]
    previewElem && (previewElem.style.backgroundColor = color)
    inputElem && (inputElem.value = color)
  }

  hide() {
    this.panel.hide()
    this.editor.$('.colorPalette-selected', this.panel.$el[0]).removeClass('colorPalette-selected')
  }
}

export default Panel
