import openFile from '../../../lib/openFile'
import attachFile from './components/attach'
import Uploader from '../../../lib/Uploader'
import deleteUploadedFile from './components/deleteUploadedFile'
import deleteAttachedFile from './components/deleteAttachedFile'

const plugin = editor => {
  const settings = editor.settings.fileUpload
  const maxSize = settings.max_size || 10 * 1024 * 1024
  const maxCount = settings.max_count || 10
  const uploader = new Uploader(editor, settings.upload_url)
  uploader.initDom()

  editor.addCommand('KfileUpload', () => {
    openFile({ multiple: true }).then(files => {
      foreachAttachFiles(files, false)
    })
  })

  editor.addButton('fileUpload', {
    icon: 'file',
    tooltip: 'upload file',
    onclick() {
      editor.execCommand('KfileUpload')
    },
    stateSelector: 'a:not([data-mce-object],[data-mce-placeholder],figure[data-ke-type="file"])',
  })

  editor.on('KeyDown', e => {
    if ((e.keyCode == 8 || e.keyCode == 46) && editor.selection) {
      const selectedNode = editor.selection.getNode()
      if (selectedNode && selectedNode.nodeName == 'A') {
        const parentFigureElement = editor.dom.getParent(selectedNode, 'figure[data-ke-type="file"]')
        if (parentFigureElement) {
          e.preventDefault()
          deleteUploadedFile(selectedNode, settings, editor)
          deleteAttachedFile(editor, parentFigureElement)
        }
      }
    }
  })

  editor.on('drop', e => {
    if (
      e &&
      e.dataTransfer &&
      e.dataTransfer.files.length > 0 &&
      e.dataTransfer.types[e.dataTransfer.types.length - 1] == 'Files'
    ) {
      const files = e.dataTransfer.files
      const notImageFiles = []
      for (let i = 0; i < files.length; i++) {
        let file = files[i]
        if (file.type.indexOf('image/') < 0) {
          notImageFiles.push(file)
        }
      }
      foreachAttachFiles(notImageFiles, true)
    }
  })

  const attachedFilePredicate = element => {
    return editor.dom.is(element, 'a[data-ke-type="a-file"]')
  }

  editor.addButton('deleteFile', {
    title: 'title test',
    icon: 'file',
    tooltip: 'test',
    cmd: 'deleteFile',
  })

  editor.addCommand('deleteFile', () => {
    const selectedNode = editor.selection.getNode()
    const parentFigureElement = editor.dom.getParent(selectedNode, 'figure[data-ke-type="file"]')

    deleteUploadedFile(selectedNode, settings, editor)
    deleteAttachedFile(editor, parentFigureElement)
  })

  editor.addContextToolbar(attachedFilePredicate, 'deleteFile')

  const getMegaBytes = number => {
    return (number / (1024 * 1024)).toFixed(2) + 'MB'
  }

  const uploadedCallback = resultObj => {
    if (resultObj.data.url) {
      const url = resultObj.data.url
      const file = resultObj.options.file
      const $fileNode = editor.$('#' + resultObj.options.targetId, editor.getBody())
      const sizeNode = editor.$('.size', $fileNode[0])[0]
      const $cannelButton = editor.$('.cancel', $fileNode[0])
      uploader.removeProgressBar(resultObj.options)
      if ($fileNode[0]) {
        $fileNode.attr('id', null)
        $fileNode.attr('href', url)
        $fileNode.attr('data-mce-href', url)
        sizeNode.innerHTML = getMegaBytes(file.size)
      }
      $fileNode.removeClass('uploading')
      $cannelButton.remove()
    } else {
      alert('업로드가 실패했습니다. 네트워크 상태를 확인해 주세요.')
      const $fileNode = editor.$('#' + resultObj.options.targetId, editor.getBody())
      $fileNode.closest('figure').remove()
    }
  }

  const foreachAttachFiles = (files, isDropAction = false) => {
    const length = files.length
    if (length > maxCount) {
      alert(`업로드가 실패했습니다. 최대 ${maxCount}개까지 업로드할 수 있습니다.`)
      return false
    }
    for (let file of files) {
      if (file.size > maxSize) {
        alert(`업로드에 실패했습니다. 파일당 10MB까지 업로드할 수 있습니다`)
        return false
      }
    }
    const failedResult = []
    let i = 0
    for (let file of files) {
      if (file.size > maxSize) {
        alert(`업로드가 실패했습니다. 10MB까지 업로드할 수 있습니다.`)
        return false
      } else {
        if (file.type.indexOf('image/') == 0 && isDropAction) {
          continue
        }
        const fileId = attachFile(editor, file)
        const $fileNode = editor.$('#' + fileId, editor.getBody())
        const $fileItem = $fileNode.closest('figure')
        $fileItem.attr('data-mce-selected', null)
        const $fileNameNode = editor.$('.filename', $fileNode[0]).html('')
        const $cannelButton = editor.$('.cancel', $fileNode[0])
        const fileNameArray = file.name.split('.')
        let $nameNode, $typeNode

        $cannelButton.on('click', () => {
          $fileItem.remove()
        })

        if (fileNameArray.length > 1) {
          const typeString = '.' + (fileNameArray[fileNameArray.length - 1] || '')
          $typeNode = editor.$(`<span class="type">${typeString}</span>`)
          fileNameArray.pop()
          $nameNode = editor.$(`<span class="name">${fileNameArray.join('.')}</span>`)
        } else {
          $nameNode = editor.$(`<span class="name">${file.name}</span>`)
        }

        $fileNameNode.append($nameNode)
        $fileNameNode.append($typeNode)

        const $sizeNode = editor.$('.size', $fileNode[0])
        $sizeNode.html('')
        const $loadeSizeNode = editor.$('<span class="loaded"></span>')
        const $totalSizeNode = editor.$(`<span class="total"> / ${getMegaBytes(file.size)}</span>`)
        $sizeNode.append($loadeSizeNode)
        $sizeNode.append($totalSizeNode)
        uploader.addProgressBar({ targetId: fileId })
        const $progressbarNode = editor.$('.bar_progress', $fileItem[0])
        uploader.uploadForFile({
          file: file,
          targetId: fileId,
          isInvisible: true,
          callback: resultObj => {
            i++
            uploadedCallback(resultObj)
            if (i === length && failedResult.length) {
              displayErrorMessage(failedResult)
            }
          },
          progressHandler: (loaded, total) => {
            $loadeSizeNode.html(getMegaBytes(loaded))
            const percent = Math.round((loaded / total) * 100)
            $progressbarNode.css('width', percent + '%')
          },
        })
      }
    }
  }

  const displayErrorMessage = failedResult => {
    window.alert('아래 파일들의 업로드가 실패하였습니다.\n\n' + failedResult.map(file => file.name).join('\n'))
  }
}

export default plugin
