import Tools from 'tinymce/core/api/util/Tools';

const getFirstFont = function (fontFamily) {
  return fontFamily ? fontFamily.split(',')[0] : '';
};

const toFontItems = function (value) {
  return value.replace(/["']/g, '').toLowerCase().split(',')
};

const findMatchingValue = function (fontItems, fontFamily) {
  const fonts = fontFamily ? toFontItems(fontFamily) : [];
  let value, text;

  if (fonts.length == 0) {
    return { value, text };
  }

  const fontItem = fontItems.find(item => !!item.fonts.find(font => fonts.includes(font)))
  if (fontItem) {
    value = fontItem.value;
    text = fontItem.text;
  }

  return {value, text};
};

const createFontNameListBoxChangeHandler = function (editor, items) {

  const fontItems = items.map(item => {
    return {
      fonts: toFontItems(item.value),
      value: item.value,
      text: item.text
    }
  })

  return function () {
    const self = this;

    // We need to remove the initial value since since the display text will
    // not be updated if we set it to the same initial value on post render.
    self.state.set('value', null);

    editor.on('init nodeChange', function (e) {
      const fontFamily = editor.queryCommandValue('FontName');
      const match = findMatchingValue(fontItems, fontFamily);
      self.value(match.value ? match.value : items[0].value);
      self.text(match.text ? match.text : items[0].text);

      if (!match && fontFamily) {
        self.text(getFirstFont(fontFamily));
      }
    });
  };
};

const createFormats = function (formats) {
  formats = formats.replace(/;$/, '').split(';');

  let i = formats.length;
  while (i--) {
    formats[i] = formats[i].split('=');
  }

  return formats;
};

const getFontItems = function (editor) {

  const defaultFontsFormats = (
    'Andale Mono=andale mono,monospace;' +
    'Arial=arial,helvetica,sans-serif;' +
    'Arial Black=arial black,sans-serif;' +
    'Book Antiqua=book antiqua,palatino,serif;' +
    'Comic Sans MS=comic sans ms,sans-serif;' +
    'Courier New=courier new,courier,monospace;' +
    'Georgia=georgia,palatino,serif;' +
    'Helvetica=helvetica,arial,sans-serif;' +
    'Impact=impact,sans-serif;' +
    'Symbol=symbol;' +
    'Tahoma=tahoma,arial,helvetica,sans-serif;' +
    'Terminal=terminal,monaco,monospace;' +
    'Times New Roman=times new roman,times,serif;' +
    'Trebuchet MS=trebuchet ms,geneva,sans-serif;' +
    'Verdana=verdana,geneva,sans-serif;' +
    'Webdings=webdings;' +
    'Wingdings=wingdings,zapf dingbats'
  );

  const fonts = createFormats(editor.settings.font_formats || defaultFontsFormats);

  return Tools.map(fonts, function (font) {
    return {
      text: { raw: font[0] },
      value: font[1],
      classes: 'font-item',
      textStyle: font[1].indexOf('dings') === -1 ? 'font-family:' + font[1] : ''
    };
  });
};

const plugin = editor => {

  const items = getFontItems(editor);
  const align = editor.settings.font_formats_align || 'left'

  editor.addButton('fontSelect', {
    type: 'listbox',
    text: 'Font Family',
    tooltip: 'Font Family',
    menu: {
      items,
      onPostRender(e) {
        e.control.fixed(true);
      }
    },
    fixedWidth: true,
    onPostRender: createFontNameListBoxChangeHandler(editor, items),
    onselect (e) {
      if(e.control.settings.value) {
        editor.execCommand('FontName', false, e.control.settings.value);
      }
    },
    onShowMenu() {
      if (align === 'center') {
        this.menu.moveRel(this.getEl(), ['bc-tc', 'tc-bc']);
      }
    }
  })
}

export default plugin;
