import axios from 'axios'

const plugin = editor => {
  let isShowPannel: boolean = false
  const grammarPannelHeight: number = 0
  const selectorContainer = editor.settings.selector
    ? document.querySelector(editor.settings.selector).parentNode
    : document.getElementById(editor.settings.id).parentNode
  const defaultCheckNodeSelector: string = 'p, h1, h2, h3, h4, h5, h6'
  const scrollContainer: any = editor.settings.scroll_container_id
    ? document.getElementById(editor.settings.scroll_container_id)
    : window

  if (!editor.settings.grammarCheck || !editor.settings.grammarCheck.check_url) {
    return
  }
  const settings: any = editor.settings.grammarCheck

  const grammarCheckUrl: string = settings.check_url
  const checkNodeSelector: string = settings.check_node_selector || defaultCheckNodeSelector
  const availableCheckNodeSelector: string = makeCheckSelector(checkNodeSelector)
  const handleStart: Function = settings.on_start || function() {}
  const handleFinish: Function = settings.on_finish || function() {}

  type nodeControl = (node: Node) => void

  const ErrorDesc = {
    spell: '철자 오류입니다.',
    space: '띄어쓰기 오류입니다.',
    space_spell: '철자와 띄어쓰기 오류입니다.',
    doubt: '맞춤법 오류가 의심됩니다.',
  }

  function makeCheckSelector(selector: string): string {
    const selectorArray: string[] = selector.split(',')
    const newArray = selectorArray.map(value => {
      value = value.trim()
      if (value.toLowerCase() == 'p') {
        return 'p:not([class])'
      } else {
        return value
      }
    })
    return newArray.join(', ')
  }

  function getCheckingText(): string {
    const $ = editor.dom.$
    let textStr: string = '',
      $target
    $(availableCheckNodeSelector).each(function(index, node) {
      if (index > 0) {
        textStr += '\n'
      }
      if (node.nodeName == 'OL' || node.nodeName == 'UL') {
        if (node.parentNode.nodeName == 'BODY') {
          textStr += $(node).text()
        }
      } else {
        $target = $(node.cloneNode(true))
        textStr += $target.text()
      }
    })
    return textStr
  }

  function getCheckTargetTextNodeList(): Array<Node> {
    const checkTextNodeList: Array<Node> = []
    function pushTextNode(node: any, textNodeArray: Array<Node>) {
      if (node.nodeType == 1) {
        for (let childNode of node.childNodes) {
          pushTextNode(childNode, textNodeArray)
        }
      } else if (node.nodeType == 3) {
        textNodeArray.push(node)
      }
    }

    editor.dom.$(availableCheckNodeSelector).each(function(index, node) {
      pushTextNode(node, checkTextNodeList)
    })

    return checkTextNodeList
  }

  function setSelection(
    startNode: Node,
    startOffset: number = 0,
    endNode: Node = startNode,
    endOffset: number = startOffset
  ): void {
    const range = editor.dom.createRng()
    range.setStart(startNode, startOffset)
    range.setEnd(endNode, endOffset)
    editor.selection.setRng(range)
  }

  function clearSelection(): void {
    editor.selection.getSel().removeAllRanges()
  }

  function processGrammar(checkedResult): void {
    const errorWordList = []
    const sentences = checkedResult.sentences
    for (let sentence of sentences) {
      for (let word of sentence.result) {
        if (word.etype != 'no_error') {
          errorWordList.push(word)
        }
      }
    }
    if (errorWordList.length > 0) {
      markingErrorText(errorWordList)
    } else {
      alert('맞춤법 오류를 찾지 못했습니다.')
    }
  }

  function markingErrorText(errorWordList): void {
    const $ = editor.dom.$
    handleStart()
    editor.undoManager.transact(function() {
      const provenDataList = getProvenGrammarData(errorWordList)
      for (let i = provenDataList.length; i--; ) {
        let provenData = provenDataList[i]
        if (provenData.startNode && provenData.endNode) {
          setSelection(provenData.startNode, provenData.startOffset, provenData.endNode, provenData.endOffset)
          editor.formatter.register('findGrammarErrorFormat', {
            inline: 'span',
            attributes: {
              'data-grammar': JSON.stringify(provenData.result),
              'data-grammar-id': 'grammar' + i,
              'data-grammar-focus': i === 0 ? 'true' : 'false',
            },
          })
          editor.formatter.apply('findGrammarErrorFormat')
        }
      }
      clearSelection()
      addClickEvent()
      const focusNode = $('span[data-grammar-focus="true"]')[0]
      if (activeGrammarSentence(focusNode)) {
        editor.getBody().setAttribute('contenteditable', 'false')
      }
    })
  }

  function closeGrammerCheck(): void {
    const $ = editor.dom.$
    editor.undoManager.ignore(function() {
      $('span[data-grammar]').each(function(index, node) {
        $(node.firstChild).unwrap()
      })

      $('#grammarPannel', document.body).hide()

      selectorContainer.style.marginBottom = ''
      isShowPannel = false
      editor.getBody().setAttribute('contenteditable', 'true')
    })
    editor.undoManager.data.pop()
    editor.undoManager.add()
    handleFinish()
  }

  function addClickEvent(): void {
    const $ = editor.dom.$
    $('span[data-grammar]').on('click', function(e) {
      activeGrammarSentence(e.target)
    })
  }

  function activeGrammarSentence(node: Node): boolean {
    const $ = editor.dom.$
    const $node = $(node)
    if ($node.attr('data-grammar')) {
      const isFoucs: boolean = $node.attr('data-grammar-focus') == 'true'
      if (!isFoucs) {
        $('span[data-grammar-focus="true"]').attr('data-grammar-focus', 'false')
        const grammarId = $node.attr('data-grammar-id')
        $(`span[data-grammar-id="${grammarId}"]`).attr('data-grammar-focus', 'true')
      }
      moveToScroll(node)
      showReplacePannel(node)
      return true
    }
    return false
  }

  const moveToScroll: nodeControl = node => {
    if (scrollContainer == window) {
      const nodeRect = editor.dom.getRect(node)
      const containerTop = editor.getContainer().offsetTop
      const screenscrollGap = grammarPannelHeight + containerTop
      const scrollY = window.scrollY || window.pageYOffset
      if (nodeRect.y + containerTop < scrollY || nodeRect.y + screenscrollGap > window.innerHeight + scrollY) {
        window.scrollTo(0, nodeRect.y)
      }
    } else {
      const nodeRect = editor.dom.getRect(node)
      scrollContainer.scrollTop = nodeRect.y
    }
  }

  const showReplacePannel: nodeControl = node => {
    const $ = editor.dom.$
    const $node = $(node)
    const data = JSON.parse($node.attr('data-grammar'))
    const grammarPannel = document.getElementById('grammarPannel')

    if (!isShowPannel) {
      selectorContainer.style.marginBottom = grammarPannelHeight + 'px'
      if (grammarPannel) {
        setDataForGrammarPannel(grammarPannel, data, node)
        grammarPannel.style.display = 'block'
      } else {
        const resultStr = `<div id="grammarPannel" class="grammar_wrapper">
        <p class="grammar_result_detail"><span class="grammar_current_text">맞춤법 검사</span><span class="grammar_current_count"></span><span>/</span><span class="grammar_total_count"></span></p>
        <div class="grammar_result">
          <input class="grammar_wanna_word" type="text" data-grammar-pannel-output value="">
          <span class="grammar_error_type" data-grammar-pannel-etype></span>
          <button class="btn grammar_check_next" data-grammar-turn>넘기기</button>
          <button class="btn btn-default grammar_check_replace" data-grammar-replace>수정</button>
        </div>
        <button class="grammar_close" data-grammar-close>종료</button>
        </div>`
        const $grammarNode = $(resultStr)
        setDataForGrammarPannel($grammarNode[0], data, node)
        $('[data-grammar-replace]', $grammarNode[0]).on('click', replaceSentence)
        $('[data-grammar-turn]', $grammarNode[0]).on('click', function(e) {
          let $focusNode = $('span[data-grammar-focus="true"]')
          focusNextGrammar(
            $focusNode
              .attr('data-grammar-id')
              .split('grammar')
              .join('')
          )
        })
        $('[data-grammar-close]', $grammarNode[0]).on('click', closeGrammerCheck)
        $(document.body).append($grammarNode)
      }
      isShowPannel = true
    } else {
      setDataForGrammarPannel(grammarPannel, data, node)
    }
  }

  function setDataForGrammarPannel(pannelNode: Node, data, grammarNode: Node): void {
    const $ = editor.dom.$
    const $grammarNodes = $('span[data-grammar]')
    let currentCount: number = 0
    for (let node of $grammarNodes) {
      currentCount++
      if (node == grammarNode) {
        break
      }
    }

    const $outputInputNode = $('[data-grammar-pannel-output]', pannelNode)
    const $etypeSpanNode = $('[data-grammar-pannel-etype]', pannelNode)
    const $currentCountNode = $('.grammar_current_count', pannelNode)
    const $totalCountNode = $('.grammar_total_count', pannelNode)
    $outputInputNode[0].value = data.output
    if (ErrorDesc[data.etype]) {
      $etypeSpanNode.text(ErrorDesc[data.etype])
    }
    $currentCountNode.text(currentCount)
    $totalCountNode.text($grammarNodes.length)
  }

  function replaceSentence(): void {
    const $ = editor.dom.$
    const $focusNode = $('span[data-grammar-focus="true"]')
    if ($focusNode.length > 0) {
      const replaceText: string = $('[data-grammar-pannel-output]', document.body)[0].value
      $focusNode.each(function(index, node) {
        if (index == 0) {
          let firstNode = node.firstChild
          while (node && node.nodeType != 3) {
            node && (node = node.firstChild)
            while (node.nextSibling) {
              $(node.nextSibling).remove()
            }
          }
          if (node.nodeType == 3) {
            node.nodeValue = replaceText
          } else {
            node.innerHTML = replaceText
          }
          $(firstNode).unwrap()
        } else {
          $(node).remove()
        }
      })
      focusNextGrammar(
        $focusNode
          .attr('data-grammar-id')
          .split('grammar')
          .join('')
      )
    }
  }

  function focusNextGrammar(currentIndexId: number) {
    const $ = editor.dom.$
    const $grammarNodes = $('span[data-grammar]')
    if ($grammarNodes.length == 0) {
      alert('맞춤법 검사를 완료했습니다.')
      closeGrammerCheck()
    } else {
      for (let node of $grammarNodes) {
        let nodeGrammarId: number = node
          .getAttribute('data-grammar-id')
          .split('grammar')
          .join('')
        if (Number(currentIndexId) < Number(nodeGrammarId)) {
          let nextNode = $(`span[data-grammar-id="grammar${nodeGrammarId}"]`)
          activeGrammarSentence(nextNode[0])
          return
        }
      }
      activeGrammarSentence($grammarNodes[0])
    }
  }

  function getProvenGrammarData(errorWordList) {
    const textNodeList = getCheckTargetTextNodeList()

    const provenResultArray = []
    let textNodeIndex: number = 0,
      fromIndex: number = 0,
      isResetFromIndex: boolean = false
    let errorInput: string, isNextTextCheck: boolean, mergedCheckText: string, textNode, indexOfStart: number
    for (let errorWordObj of errorWordList) {
      errorInput = errorWordObj.input
      isNextTextCheck = true
      mergedCheckText = ''
      const provenData = {
        result: errorWordObj,
        startNode: null,
        startOffset: 0,
        endNode: null,
        endOffset: 0,
      }
      while (isNextTextCheck && textNodeIndex < textNodeList.length) {
        textNode = textNodeList[textNodeIndex]
        mergedCheckText += textNode.textContent
        indexOfStart = mergedCheckText.indexOf(errorInput, fromIndex)
        if (indexOfStart != -1) {
          isNextTextCheck = false
          provenData.endNode = textNode
          provenData.endOffset = indexOfStart + errorInput.length + textNode.length - mergedCheckText.length
          if (provenData.endOffset < 0) {
            provenData.endOffset += mergedCheckText.length - textNode.length
          }
          let startFromText: string = ''
          let backFlowStartIndexOf: number
          for (let i = textNodeIndex; i >= 0; i--) {
            startFromText = textNodeList[i].textContent + startFromText
            backFlowStartIndexOf = startFromText.indexOf(errorInput)
            if (backFlowStartIndexOf != -1) {
              provenData.startOffset = backFlowStartIndexOf
              provenData.startNode = textNodeList[i]
              break
            }
          }
          if (mergedCheckText.length <= indexOfStart + errorInput.length) {
            textNodeIndex++
            fromIndex = 0
            isResetFromIndex = true
          }
        } else {
          textNodeIndex++
          // 앞 text와 merge를 했을때 처음부터 찾을때는 0
          // fromIndex = 0
        }
      }
      if (!isResetFromIndex) {
        fromIndex = provenData.endOffset
      }
      provenResultArray.push(provenData)
    }
    return provenResultArray
  }

  function grammarCheck(): void {
    const textContent: string = getCheckingText()
    const data: { q: string } = { q: textContent }
    axios
      .post(grammarCheckUrl, data)
      .then(function(res) {
        return res.data
      })
      .then(function(result) {
        processGrammar(result)
      })
      .catch(function(res) {})
  }

  /**
   * body[contenteditable="false"] mouseup 시
   * body 밑에 &#65279; 생성 방지
   * 스크롤 최하단에서 본문 클릭 시 해당 요소로 화면이 움직임
   */

  editor.on('mousedown mouseup click', function(e) {
    if (editor.getBody().getAttribute('contenteditable') == 'false') {
      e.preventDefault()
      return false
    }
  })

  editor.addCommand('grammarCheck', grammarCheck)
  editor.addCommand('grammarReplaceSentence', replaceSentence)

  editor.addButton('grammarCheck', {
    icon: 'code',
    tooltip: 'Grammar check',
    onclick: grammarCheck,
  })
}

export default plugin
