import openFile from 'src/lib/openFile'
import Uploader from 'src/lib/Uploader'
import Layer from '../ui/Layer'
import ResizePanel from '../ui/ResizePanelAction'

const plugin = editor => {
  const _kImageSettings = editor.settings.kImage
  const maxFileSize = _kImageSettings.max_size || 10 * 1024 * 1024
  const maxCount = 15
  const uploader = new Uploader(editor, _kImageSettings.upload_url, _kImageSettings.upload_handler)
  const layerWindowId = 'imageSliceWindow'
  const layer = new Layer(editor, layerWindowId)
  let isUploading = false

  const imageUploadCallback = resultObj => {
    if (resultObj.data) {
      layer.appendImageShow(resultObj)
    }
    layer.setIsUploading(false)
  }

  const uploadImage = file => {
    layer.setIsUploading(true)
    const imageId = 'imageId_' + file.lastModified
    layer.appendImageWrap(imageId)
    uploader.uploadForFile({
      file: file,
      targetId: imageId,
      callback: imageUploadCallback,
      isInvisible: true,
    })
  }

  const foreachAttachImages = files => {
    if (files.length > maxCount) {
      alert(`업로드가 실패했습니다. 이미지는 한번에 ${maxCount}장까지 업로드할 수 있습니다.`)
      return false
    }
    for (let file of files) {
      if (file.size > maxFileSize) {
        alert(`업로드에 실패했습니다. 파일당 10MB까지 업로드할 수 있습니다`)
        return false
      } else if (file.type.indexOf('image/') < 0) {
        alert(`업로드에 실패했습니다. 이미지 파일만 업로드할 수 있습니다`)
        return false
      }
    }
    const fileLength = files.length > maxCount ? maxCount : files.length
    let isShowLayer: boolean = !!document.getElementById(layerWindowId)
    let isActiveLayer = false
    for (let i = 0, l = fileLength; i < l; i++) {
      if (files[i].type.indexOf('image/') < 0) {
        continue
      } else if (files[i].size > maxFileSize) {
        alert(`업로드가 실패했습니다. 10MB까지 업로드할 수 있습니다.`)
        return false
      }
      if (!isShowLayer && !isActiveLayer) {
        layer.show()
        isActiveLayer = true
      }
      uploadImage(files[i])
    }
    window.setTimeout(() => {
      editor.execCommand('mceAutoResize')
    })
    return true
  }

  editor.addCommand('AttachSlideshowImage', foreachAttachImages)
  editor.addCommand('KImageSlide', function() {
    openFile({ multiple: true, accept: 'image/*' }).then(files => {
      foreachAttachImages(files)
    })
  })

  editor.addButton('imageSlide', {
    icon: 'imageSlide',
    tooltip: 'imageSlide',
    cmd: 'KImageSlide',
  })

  const predicate = element => {
    return editor.dom.is(element, 'figure[data-ke-type="imageSlide"]')
  }

  const getFigure = element => {
    const $figureNode = editor.$(element).closest('figure[data-ke-type="imageSlide"]')
    if ($figureNode && $figureNode[0]) {
      return $figureNode
    } else {
      return null
    }
  }

  const postrender = function(e) {
    const control = this
    let $figureNode, keStyle, textAlignValue, state
    $figureNode = getFigure(editor.selection.getNode())

    const toggleActive = $figureNode => {
      keStyle = $figureNode.attr('data-ke-style')
      control.active(control.settings.activestate == keStyle)
    }

    if ($figureNode) {
      toggleActive($figureNode)
    }

    editor.on('nodechange', e => {
      const selectNode = editor.selection.getNode()
      $figureNode = getFigure(selectNode)
      if ($figureNode) {
        toggleActive($figureNode)
      }
    })
  }

  editor.addButton('imageSlideEdit', {
    title: 'imageSlide edit',
    icon: 'imageslideedit',
    tooltip: 'Add/Delete Image',
    onclick(e) {
      const slideItem = editor.$(editor.selection.getNode()).closest('figure')
      const data = JSON.parse(slideItem[0].dataset.images)
      data.reverse()
      layer.show(slideItem[0])
      data.forEach(image => {
        layer.appendImage({
          data: {
            url: image.src,
            caption: image.caption,
          },
        })
      })
    },
  })

  editor.addButton('kImageSlideAlignLeft', {
    title: 'Align left',
    cmd: 'kImageAlignLeft',
    icon: 'imagealignleft',
    activestate: 'alignLeft',
    onpostrender: postrender,
  })

  editor.addButton('kImageSlideAlignCenter', {
    title: 'Align center',
    cmd: 'kImageAlignCenter',
    icon: 'imagealigncenter',
    activestate: 'alignCenter',
    onpostrender: postrender,
  })

  editor.addButton('kImageSlideAlignRight', {
    title: 'Align right',
    cmd: 'kImageAlignRight',
    icon: 'imagealignright',
    activestate: 'alignRight',
    onpostrender: postrender,
  })

  const panel = new ResizePanel(editor, 'kImageSlideResizeWidthInput')

  editor.addButton('kImageSlideResize', {
    title: 'Resize Image',
    icon: 'resizeimage',
    type: 'panelbutton',
    tooltip: 'Resize Image',
    panel: {
      classes: 'kImageResize tinymce',
      role: 'dialog',
      fixed: false,
      onPostRender() {
        panel.init(this)
      },
      items: [
        { type: 'label', classes: 'widthLabel', text: 'W' },
        {
          type: 'textbox',
          id: 'kImageSlideResizeWidthInput',
          classes: 'kImageWidthInput',
          size: 3,
          maxLength: 3,
          label: 'Width',
          name: 'width',
          onsubmit: function() {
            panel.updateImageWidth(this.value())
            this.parent().hide()
          },
        },
        {
          type: 'button',
          classes: 'kImageMaxSize',
          title: 'Resize image Max size',
          icon: 'reiszeimagemax',
          tooltip: 'Max size',
          onclick: panel.updateMaxSize.bind(panel),
        },
      ],
    },
    onshow(e: any) {
      e.control.fixed(false)
      panel.show()
    },
    onhide() {
      editor.focus()
    },
  })

  editor.addContextToolbar(
    predicate,
    'imageSlideEdit kImageSlideResize | kImageSlideAlignLeft kImageSlideAlignCenter kImageSlideAlignRight'
  )

  editor.on('PreProcess', e => {
    const $ = editor.dom.$
    $('[data-ke-type="imageSlide"] .mark_slide', e.node).each((idx, elm) => {
      $(elm).remove()
    })
  })
}

export default plugin
