import attachLocalImage from './components/attachLocalImage'
import openFile from '../../../lib/openFile'
import Caption from './components/Caption'
import ContextToolbar from './components/ContextToolbar'
import KeyEvent from './components/KeyEvent'
import Uploader from '../../../lib/Uploader'
import Grid from './components/Grid'
import bowser from 'bowser'
import Delay from 'tinymce/core/api/util/Delay'
import RangeUtils from 'tinymce/core/api/dom/RangeUtils'

const plugin = editor => {
  const _editingImageId = 'kEditorPhotosEditingImage'
  const _kImageSettings = editor.settings.kImage
  const contextToolbar = new ContextToolbar(editor)
  const maxFileSize = _kImageSettings.max_size || 10 * 1024 * 1024
  const maxCount = _kImageSettings.max_count || 50
  const uploader = new Uploader(editor, _kImageSettings.upload_url, _kImageSettings.upload_handler)

  editor.addCommand('KImageUploadUrl', uploader.execUploadForOption)
  editor.addCommand('KImageUploadFile', uploader.execUploadForOption)

  const imageUploadCallback = resultObj => {
    if (resultObj && resultObj.data && resultObj.data.url) {
      const $imageNode = editor.dom.$('#' + resultObj.options.targetId)
      const imageOffsetWidth = $imageNode[0].offsetWidth
      const uploadData = resultObj.data
      if ($imageNode[0] && uploadData) {
        uploader.removeProgressBar(resultObj.options)
        if (bowser.msie) {
          $imageNode[0].outerHTML = `<img width="${imageOffsetWidth}" src="${uploadData.url}" data-filename="${
            uploadData.name
          }">`
        } else {
          $imageNode.attr('data-mce-src', null)
          $imageNode.attr('src', uploadData.url)
          $imageNode.attr('data-filename', uploadData.name)
          $imageNode.attr('class', null)
          $imageNode.attr('id', null)
          $imageNode.attr('style', null)
          $imageNode.attr('data-mce-style', null)
        }
        editor.undoManager.add()
      }
    } else {
      alert('업로드가 실패했습니다. 네트워크 상태를 확인해 주세요.')
      const $imageNode = editor.dom.$('#' + resultObj.options.targetId)
      $imageNode.closest('figure[data-ke-type="image"]').remove()
    }
    editor.undoManager.ignore(() => {
      editor.execCommand('editImageForPhotosEnd')
    })
  }

  const photoEditCancelCallback = () => {
    editor.undoManager.ignore(() => {
      const imageNode = editor.dom.$('#' + _editingImageId)
      if (imageNode) {
        imageNode.attr('id', null)
      }
      editor.undoManager.ignore(() => {
        editor.execCommand('editImageForPhotosEnd')
      })
    })
  }

  const photosAppSuccessCallback = (resultArr, file) => {
    uploader.uploadForFile({
      file: file,
      targetId: _editingImageId,
      callback: imageUploadCallback,
    })
  }

  const attachImage = (file, settings) => {
    const imageId = attachLocalImage(editor, file, settings)
    uploader.uploadForFile({
      file: file,
      targetId: imageId,
      callback: imageUploadCallback,
    })
  }

  const foreachAttachImages = files => {
    if (files.length > maxCount) {
      alert(`업로드가 실패했습니다. 이미지는 한번에 ${maxCount}장까지 업로드할 수 있습니다.`)
      return false
    }
    for (let file of files) {
      if (file.size > maxFileSize) {
        alert(`업로드에 실패했습니다. 파일당 10MB까지 업로드할 수 있습니다`)
        return false
      } else if (file.type.indexOf('image/') < 0) {
        alert(`업로드에 실패했습니다. 이미지 파일만 업로드할 수 있습니다`)
        return false
      }
    }
    const fileLength = files.length > maxCount ? maxCount : files.length
    for (let i = 0, l = fileLength; i < l; i++) {
      if (files[i].type.indexOf('image/') < 0) {
        continue
      } else if (files[i].size > maxFileSize) {
        alert(`업로드가 실패했습니다. 10MB까지 업로드할 수 있습니다.`)
        return false
      }
      attachImage(files[i], _kImageSettings)
    }
    Delay.setTimeout(() => {
      editor.execCommand('mceAutoResize')
    })
  }

  const getCaretRangeFromEvent = function(editor, e: MouseEvent) {
    return RangeUtils.getCaretRangeFromPoint(e.clientX, e.clientY, editor.getDoc())
  }

  const setFocusedRange = function(editor, rng) {
    editor.focus()
    editor.selection.setRng(rng)
  }

  editor.on('dragover dragend', function(e) {
    e.preventDefault()
    setFocusedRange(editor, getCaretRangeFromEvent(editor, e))
  })

  editor.on('drop', e => {
    const rng = getCaretRangeFromEvent(editor, e)
    if (e.target.nodeName == 'TD') {
      e.preventDefault()
      e.stopPropagation()
      return false
    }
    if (
      e &&
      e.dataTransfer &&
      e.dataTransfer.files.length > 0 &&
      e.dataTransfer.types[e.dataTransfer.types.length - 1] == 'Files'
    ) {
      const files = e.dataTransfer.files
      const imageFiles = []
      e.preventDefault()
      e.stopPropagation()
      setFocusedRange(editor, rng)
      for (let file of files) {
        if (file.type.indexOf('image/') >= 0) {
          imageFiles.push(file)
        }
      }
      foreachAttachImages(imageFiles)
    } else {
      const $selectedNode = editor.$(editor.selection.getNode())
      if (e.target && $selectedNode[0].nodeName == 'IMG') {
        e.preventDefault()
        e.stopPropagation()
      }
    }
  })

  editor.on('PreProcess', e => {
    editor.dom.$('[data-ke-type="image"] img', e.node).each((idx, elm) => {
      if (elm.id) {
        throw new Error('이미지 업로드가 완료된 후 시도해 주세요.')
      }
    })
  })

  editor.addButton('kImage', {
    icon: 'image',
    tooltip: '이미지',
    onclick() {
      onButtonClicked()
    },
  })

  const onButtonClicked = () => {
    openFile({ multiple: true, accept: 'image/*' }).then(files => {
      foreachAttachImages(files)
    })
  }

  editor.addCommand('KImageUpload', onButtonClicked)
  editor.addCommand('KImageMobile', onButtonClicked)

  KeyEvent.register(editor)

  editor.on('SetContent', e => {
    let captionElm
    editor.dom
      .$('figure[data-ke-type="image"], figure[data-ke-type="imageGrid"]', e.node)
      .each((idx, elm: HTMLElement) => {
        captionElm = elm.querySelector('figcaption')
        if (bowser.msie) {
          elm.setAttribute('contenteditable', 'false')
          captionElm && captionElm.setAttribute('contenteditable', 'true')
        } else {
          elm.removeAttribute('contenteditable')
          captionElm && captionElm.removeAttribute('contenteditable')
        }
      })
  })

  editor.on('init', () => {
    uploader.initDom()

    if (_kImageSettings.is_caption) {
      new Caption(editor)
    }
    if (_kImageSettings.is_grid_layout) {
      new Grid(editor)
    }
    contextToolbar.register(photosAppSuccessCallback, photoEditCancelCallback, _editingImageId)
  })
}

export default plugin
