import regex from './regex';
import rm from './safeRemove';

const htmlParser = (editor, tree, targetNode) => {

  let imgCnt = 0;
  const imgList = [];
  const invalidChildList = editor.settings.kPaste.invalid_table_children;
  const validElements = editor.settings.kPaste.valid_elements;
  const extendedValidElements = editor.settings.kPaste.extended_valid_elements;
  const validAttributes = editor.settings.kPaste.valid_attributes;
  const validColors = editor.settings.colorPalette.color_map_preset;

  const modifyAttributes = node => {
    const clonedNode = node.cloneNode(false);
    let pos = 0;
    while(clonedNode.attributes[pos]) {
      if(!validAttributes.includes(clonedNode.attributes[pos].name)) {
        clonedNode.removeAttribute(clonedNode.attributes[pos].name);
      }
      else {
        pos++;
      }
    }
    if(clonedNode.nodeName === 'TABLE') {
      clonedNode.classList.add('mce-table');
      clonedNode.style.width = "100%";
      clonedNode.style["table-layout"] = "fixed";
    }
    return clonedNode;
  }

  const modifyOuterColor = nodeColor => {
    const span = document.createElement('span');
    if(!nodeColor) {
      return span;
    }
    validColors.forEach(color => {
      const colorString = `rgb(${parseInt(color.slice(1,3), 16)}, ${parseInt(color.slice(3,5), 16)}, ${parseInt(color.slice(5,7), 16)})`
      if(nodeColor === colorString) {
        span.style.color = nodeColor;
      }
    })
    return span;
  }

  const validate = node => {

    for(let type of validElements) {
      if(node.nodeName === type) {
        return modifyAttributes(node);
      }
    }
    if(extendedValidElements[node.nodeName]) {
      const c = node.cloneNode(false);
      const m = document.createElement(extendedValidElements[node.nodeName]);
      m.innerHTML = c.innerHTML;
      return m;
    }
    if(node.nodeName === 'DIV') {
      return document.createElement('p');
    }
    return null;
  };

  const isKeditorContent = (node) => {
    if(node.nodeType !== 1) return false;
    if(node && node.querySelector('.keditor-inner-content')) {
      return true;
    }
    return false;
  }

  const isInvalidTableChildren = node => {
    for(let type of invalidChildList) {
      if(node.nodeName === type) {
        return true;
      }
    }
    return false;
  }

  const isValidSpan = node => {
    while(node) {
      if(node.childNodes.length === 1 && ['#text', 'B', 'U', 'S', 'I'].includes(node.firstChild.nodeName)) {
        node = node.firstChild;
      }
      else if(node.nodeName === '#text') {
        return true;
      }
      else {
        return false;
      }
    }
  }

  // 에디터 내의 요소들을 복-붙할 경우 id가 겹치지 않게끔. 나중에 로직을 보완해야
  // 현재는 1 depth만 살펴보기 때문에 추후 바뀔 경우 이 점에 주의
  const generateId = node => {
    for(let child of node.childNodes) {
      if(!child.id) {
        continue;
      }
      child.id = child.id.replace(/_.*_?/i, `_${Date.now()}`);
    }
    return node;
  }

  const recursiveTraversal = (node, parent) => {
    if(!node) {
      return parent;
    }
    if(isKeditorContent(node)) {
      if(targetNode && targetNode === 'TABLE') {
        return recursiveTraversal(node.firstChild, parent);
      }
      return generateId(node.querySelector('.keditor-inner-content'));
    }
    if(targetNode && targetNode === 'TABLE') {
      if(isInvalidTableChildren(node)) {
        return;
      }
    }
    const curRoot = validate(node) || parent;

    if(node.nodeName === 'SCRIPT') {
      return {};
    }

    if(node.firstChild) {

      if(node.nodeName === 'SPAN' || node.nodeName === 'CODE') {
        console.log('isValidSpan', node.nodeName, isValidSpan(node));
        if(isValidSpan(node)) {
          const newSpan = modifyOuterColor(node.style.color);
          newSpan.appendChild(node.firstChild);
          parent.appendChild(newSpan);
          return {};
        }
      }

      const childArray = Array.from(node.childNodes);
      let len = node.childNodes.length;

      if(curRoot !== parent) {
        parent.appendChild(curRoot);
      }
      childArray.forEach(c => {recursiveTraversal(c, curRoot)})
    }

    else {
      if(node.nodeName === 'BR') {
        if(node.hasAttribute('data-mce-bogus')) {
          return {};
        }
        parent.appendChild(document.createElement('br'));
        return {};
      }

      if(node.nodeName === 'FIGURE' && node.getAttribute('data-ke-type')) {
        if(node.id) {
          node = generateId(node.parentNode).firstChild;
        }

        parent.appendChild(node);
        return {};
      }

      if(node.nodeName === 'IMG') {
        let curParent = node.parentNode;
        while(curParent) {
          if(curParent.nodeName === 'FIGURE' && curParent.hasAttribute('data-ke-type')) {
            break;
          }
          curParent = curParent.parentNode;
        }
        if(curParent) {
          curParent = generateId(curParent.parentNode).firstChild
          parent.appendChild(curParent);
          return {};  
        }
        else {
          let pos = 0;
          while(node.attributes[pos]) {
            if(node.attributes[pos].name !== 'src') {
              node.removeAttribute(node.attributes[pos].name);
            }
            else {
              pos++;
            }
          }
  
          const figureNode = document.createElement('figure');
          figureNode.setAttribute('data-ke-type', 'image');
          figureNode.appendChild(node);
          if (editor.settings.kImage.is_caption) {
            figureNode.appendChild(document.createElement('figcaption'));
          }
          parent.appendChild(figureNode);
          node.id = `img_${Date.now()}_${imgCnt++}`;
          imgList.push(node);
        }
      }

      if(regex.isUrlInText(node.textContent)) { // 현재 노드의 텍스트 안에 url이 포함될 경우 처리
        let innerHTMLString = node.textContent;
        innerHTMLString = innerHTMLString.replace(regex.regexString.isUrlInText, (match, ...args) => {
          return `<a href=${args[0]}>${args[0]}</a>`
        })
        parent.innerHTML += innerHTMLString;
        return {};
      }

      if(node.nodeName === '#text') {
        const textNode = document.createTextNode(node.textContent);
        parent.appendChild(textNode);
      }
    }

    if(curRoot.nodeName === 'P' && !curRoot.firstChild) {
      rm(node);
    }

    return parent;
  }

  const newRoot = document.createElement('div', {'class':'pasteroot'});
  const result = recursiveTraversal(tree, newRoot);
  return {cleanedDiv: result, imgList};
}

export default htmlParser;
