const handleCopyCut = (editor, e) => {
  const isCopy = e.type === 'copy';
  const sel = editor.selection.getSel();
  const curNode = editor.selection.getNode();
  const rng = sel.getRangeAt(0);
  const originalRange = rng.cloneRange();
  let common = rng.startContainer === rng.endContainer ? rng.commonAncestorContainer : undefined;

  // 기본적인 복사 전략은 최대한 브라우저의 네이티브 동작을 그대로 사용하되,
  // selection 관련하여 문제가 될 수 있는 부분에서 selection만 수정하는 방식

  // decoy가 필요한 이유
  // 이미지를 감싼 figure를 복사하려고 하면, 다음 형제 요소가 무엇인가에 따라 
  // range가 제대로 설정되어 있어도 figure가 아닌 img, figcaption만 클립보드에 들어오는 경우가 있음
  // 아래처럼 figure 다음 형제 요소를 안전하게 고정함으로써 문제 회피
  const getDecoy = () => {
    const decoy = editor.dom.create('p');
    decoy.textContent = "\u00a0";
    decoy.style.position = "absolute";

    return decoy;
  }

  const insertDecoy = (decoy, target) => {
    editor.dom.insertAfter(decoy, target);
  }

  // selection을 수정하는 과정에서, 에디터 내의 dom 구조가 range 수정이 용이하지 않을 경우가 있음.
  // 예컨대 표 같은 경우, 여러 셀을 선택한 것 처럼 보이더라도 실제 selection은 오직 한 개의 td 뿐이고 여러 td에 걸친 range를 만드는 것이 어려움.
  // 특히 3x3 표인데 (0,0), (1,0) 두 셀을 선택한다거나 하는 경우
  // 이런 케이스를 위해 보이지 않는 copybin을 하나 만들어서 현재 range 중 원하는 요소만을 copybin에 설정하고, 해당 copybin 내에 range를 주면 우리가 원하는대로 복사 가능.
  const set = (node, option) => {
    const copybin = prepare();
    const clonedNode = node.cloneNode(true);
    const wrapper = option.wrapper;
    const target = wrapper || clonedNode;

    if(wrapper) {
      wrapper.appendChild(clonedNode);
    }

    copybin.appendChild(target);
    rng.selectNode(target);
    editor.selection.setRng(rng);

    if(option.requireDecoy) {
      insertDecoy(getDecoy(), target);
    }

    // 네이티브 copy가 발생한 이후, copybin에 위치한 range를 다시 에디터 내로 돌려준다.
    setTimeout(() => {
      editor.selection.setRng(originalRange);

      if(!isCopy) {
        editor.dom.remove(node);
        
        if(option) {
          editor.selection.select(option.nodeToSelect);
          editor.selection.collapse(option.shouldSelectStart);
        }
      }
      editor.dom.remove(copybin);
      editor.undoManager.add();
    }, 0);
  }

  const prepare = () => {
    const copybin = document.createElement('figure');
    copybin.style.width = '1px';
    copybin.style.height = '1px';
    copybin.style.position = 'fixed';
    copybin.style.overflow = 'hidden';
    
    copybin.style.top = '50%';
    editor.getBody().appendChild(copybin);

    return copybin;
  }

  // 아래는 selection/range 수정이 필요한 경우의 처리들이다.

  // image를 감싼 figure가 선택되어야 할 경우의 조건식과 처리.
  // safari의 경우 image figure가 선택된 경우 range가 다소 다르게 잡혀서 이를 처리해주어야 한다.
  // (startContainer가 body, 혹은 더보기 영역인 경우 div[data-ke-type="moreless"]로 잡힘)
  let shouldDelayDelete = false;
  if((rng.startContainer.nodeName === 'BODY' || (rng.startContainer.nodeName === 'DIV' && rng.startContainer.getAttribute('data-ke-type') === 'moreLess')) && rng.startContainer.childNodes[rng.startOffset] === rng.endContainer) {
    common = rng.endContainer;
    shouldDelayDelete = true;
  }

  if(common && common.nodeName === 'FIGURE' && common.getAttribute('data-ke-type') === 'image') {
    const next = common.nextSibling;
    const decoy = getDecoy();
    insertDecoy(decoy, common);
        
    const nodeToSelect = !isCopy ? common.nextSibling.nextSibling || common.previousSibling : undefined;
    const collapseToStart = nodeToSelect === common.previousSibling || nodeToSelect === common.nextSibling.nextSibling;
        
    // 네이티브 copy 발생 이후 처리 (selection 복구 및 decoy 제거)
    setTimeout(() => {
      editor.dom.remove(decoy);
      if(nodeToSelect) {
        editor.selection.select(nodeToSelect);
        if(editor.dom.is(nodeToSelect, 'p, h1, h2, h3, span')) {
          editor.selection.collapse(collapseToStart);
        }
      }
    }, 0);
    
    editor.selection.select(common);

    if(!isCopy) {
      editor.execCommand('copy');

      if(shouldDelayDelete) { // safari의 경우 바로 remove를 시키면 클립보드에 들어가지 않는다.
        setTimeout(() => {
          editor.dom.remove(common);
        }, 0); 
      }
      else {
        editor.dom.remove(common);
      }
    }
  }

  // 이미지 그리드 내의 이미지 복사/잘라내기
  // 이미지 그리드의 경우 figure[data-ke-type="imageGrid"]내에 여러 사진이 들어가는 식으로 구성되어 있어서, 
  // data-ke-type="image"인 figure를 하나 만들어야 하기 때문에 에디터 영역 내에서 처리하지 않고 copybin을 만들어서 처리한다.
  if(common && common.nodeName === 'FIGURE' && common.getAttribute('data-ke-type') === 'imageGrid') {
    const newFigure = editor.dom.create('figure', {
      'data-ke-type': 'image'
    })
    set(curNode, {
      requireDecoy: true, 
      wrapper: newFigure,
    });
    if(!isCopy) {
      setTimeout(() => {
        // 잘라내기의 경우 그리드 영역 내의 사진 크기 재조정
        editor.execCommand('updateGridForImageDelete');
      }, 0);
    }
  }

  // figcaption중 일부만 선택되었을 경우, 해당 figure 전체까지 selection을 주게끔 수정.
  if(rng.endContainer.parentNode.nodeName === 'FIGCAPTION' && !common) {
    rng.setEndAfter(rng.endContainer.parentNode.parentNode);
  }
  if(rng.startContainer.parentNode.nodeName === 'FIGCAPTION' && !common) {
    rng.setStartBefore(rng.startContainer.parentNode.parentNode);
  }

  if(curNode.getAttribute('contenteditable') === 'false' && editor.dom.is(curNode, 'figure, pre, hr')) {
    const nodeToSelect = curNode.nextSibling || curNode.previousSibling;
    const shouldSelectStart = nodeToSelect === curNode.nextSibling;

    if(!isCopy) {
      editor.dom.remove(curNode);
      
      setTimeout(() => {
        editor.selection.select(nodeToSelect);
        editor.selection.collapse(shouldSelectStart);
      })
    }
  }


  // 표의 경우, 여러 셀에 걸쳐 파란색으로 selection이 된 것으로 보여도 실제 range는 하나의 td에만 적용되어 있다.
  // 따라서 일단 td가 선택되어 있는지를 확인한 뒤, 해당 표의 모든 td를 돌면서 data-mce-selected라는 어트리뷰트가 있는지 확인 후
  // 있는 셀들만 따로 cloneNode하여 새로운 테이블을 구성하고 이를 copybin에 넣어준다(원본이 바뀌면 안되므로 copybin에서 처리)
  const tdElement = editor.dom.getParent(rng.endContainer, 'td');
  const isCellSelected = tdElement && tdElement.hasAttribute('data-mce-selected');

  if(isCellSelected && rng.startContainer.nodeName === rng.endContainer.nodeName) {

    const tableElement = editor.dom.getParent(rng.endContainer, 'table').childNodes[0];
    const clonedTable = isCopy ? tableElement.cloneNode(true) : tableElement;
    const newParentTable = document.createElement('table');
    const newParentTbody = document.createElement('tbody');
    newParentTable.appendChild(newParentTbody);

    let selectedCellCount = 0, totalCellCount = 0;
    let nodeToSelect, shouldSelectStart;
    for(let row = 0; row < clonedTable.childNodes.length; row++) {
      let tr = clonedTable.childNodes[row];
      let newTrNode;
      for(let col = 0; col < tr.childNodes.length; col++) {
        totalCellCount++;
        let td = tr.childNodes[col];
        if(td.hasAttribute('data-mce-selected')) {
          selectedCellCount++;
          newTrNode = newTrNode || document.createElement('tr');
          const clonedTd = td.cloneNode(true);
          if(!clonedTd.firstChild) {
            clonedTd.appendChild(editor.dom.create('br'));
          }
          clonedTd.removeAttribute('data-mce-selected');
          clonedTd.removeAttribute('data-mce-first-selected');
          clonedTd.removeAttribute('data-mce-last-selected');
          clonedTd.setAttribute('data-mce-style', `width: ${clonedTd.style.width};`)
          newTrNode.appendChild(clonedTd);
          const tdChildNodes = Array.from(td.childNodes);
          tdChildNodes.forEach(child => {
            editor.dom.remove(child);
          });
        }
      }
      if(newTrNode) {
        newParentTbody.appendChild(newTrNode);
      }
    }

    if(selectedCellCount === totalCellCount && !isCopy) {
      nodeToSelect = tableElement.parentNode.nextSibling || tableElement.parentNode.previousSibling;
      shouldSelectStart = nodeToSelect === tableElement.parentNode.nextSibling;
      editor.dom.remove(tableElement.parentNode);
    }
    newParentTable.style['width'] = "100%";
    newParentTable.style['table-layout'] = "fixed";
    set(newParentTable, {nodeToSelect, shouldSelectStart});
  }
}

export default handleCopyCut;