import handlePaste from '../components/handlePaste';
import relocateCursor from '../../../lib/relocateCursor';
import { handlePlainText, extractPlainText } from '../components/handlePlainText';
import { handleHtml, extractHtml } from '../components/handleHtml';
import Validator from '../components/Validator';
import AttributesModifier from '../components/AttributesModifier';
import handleCopyCut from '../components/handleCopyCut';
import handleIEPaste from '../components/handleIEPaste';
import handleImage from '../components/handleImage';

const plugin = editor => {

  const modifier = new AttributesModifier(editor.settings);
  const validator = new Validator(editor.settings, modifier);

  // 붙여넣기 작업은 paste 이벤트가 preventDefault된 상태에서 일어나기 떄문에, selection이 collapse되지 않은 상태라면 이를 지워주는 것도 해야한다.
  // 지워야될 range는 이벤트 발생 당시의 range인데, 이를 바로 지울 경우 붙여넣기 작업이 오래 걸릴 경우에 동작간 시차가 생겨서 보기에 좋지 않다.
  // 따라서 원본 range를 기억하고 있는 함수를 통해 내용 삽입 직전에 삭제한다.
  const delegateDelete = rng => {
    return rangeToRestore => {
      editor.selection.setRng(rng);
      editor.undoManager.ignore(() => {
        editor.execCommand('Delete');
      })
      if(rangeToRestore) {
        editor.selection.setRng(rangeToRestore);
      }
      
      editor.nodeChanged();
    }
  }

  // selection이 collapse되지 않은 경우 해당 범위의 내용을 삭제해야 하는데, 이 때 range를 적절하게 수정해준다.
  const modifyRange = rng => {
    const { startContainer, endContainer, commonAncestorContainer, startOffset, endOffset } = rng;

    if(commonAncestorContainer.nodeName === 'FIGURE') {
      return false;
    }

    if(commonAncestorContainer.childNodes[startOffset] && commonAncestorContainer.childNodes[startOffset].nodeName === 'FIGURE' && commonAncestorContainer.childNodes[startOffset].getAttribute('data-ke-type') === 'image') {
      rng.selectNode(commonAncestorContainer.childNodes[startOffset].firstChild);
      return false;
    }

    // 만약 range가 figcaption에서 시작해 부모 figure가 아닌 다른 요소에 걸쳐있다면, range의 시작지점을 figure 다음 요소로 재설정한다.
    if(startContainer.parentNode.nodeName === 'FIGCAPTION' && commonAncestorContainer.nodeName !== '#text' && commonAncestorContainer.nodeName !== 'FIGURE') {
      rng.setStart(editor.dom.getParent(startContainer, 'body > *').nextSibling, 0);
    }

    return true;
  }

  // 붙여넣기에 사용될 dataTransferItem을 받은 뒤, 타입에 따라 html / plainText / image로 나뉘어 붙여넣기를 수행시킨다.
  const process = items => {
    const sel = editor.selection.getSel();
    let rng;

    // range는 붙여넣기 수행 중 바뀔 수 있으므로 미리 원본을 복사한다.
    if(!sel.isCollapsed) {
      rng = sel.getRangeAt(0).cloneRange();
    }
    
    items.targetNode = relocateCursor(editor, true);
    if(rng && !rng.collapsed) {
      if(modifyRange(rng)) {
        items.deleteRange = delegateDelete(rng);
      }
    }
    
    if(items.htmlItem) {
      extractHtml(items.htmlItem).then(item => handleHtml(editor, item, items.targetNode, items.deleteRange, validator, modifier));
    }
    else if(items.imageItem) {
      handleImage(editor, items.imageItem, items.deleteRange);
    }
    else if(items.plainTextItem) {
      extractPlainText(items.plainTextItem).then(item => handlePlainText(editor, item, items.targetNode, items.deleteRange));
    }
  }

  const kPasteInit = () => {

    // 테스트 코드에서 붙여넣기를 테스트하기 위한 커맨드
    // editor.execCommand('insertClipboardContent', false, 'some html string')과 같이 사용할 수 있다.
    editor.addCommand('insertClipboardContent', (ui, value) => {
      process(value);
    });

    editor.on('BeforeSetContent', e => {
      if(!e.set) {
        const isEventFromPaste = e.paste === 'kPaste' && !e.set;
        relocateCursor(editor, isEventFromPaste);
      }
    });

    // ie의 경우 다른 붙여넣기 로직을 타기 때문에 별도로 처리한다.
    if(window.tinymce.Env.ie) {
      editor.on('paste', e => {
        handleIEPaste(e, editor, process);
      });
    }
    else {
      editor.on('paste', e => {
        handlePaste(e, editor, process);
      });
    }

    editor.on('copy cut', e => {
      handleCopyCut(editor, e);
    });
  }

  kPasteInit();
}

export default plugin;