import Tools from 'tinymce/core/api/util/Tools'
import RangeWalk from 'tinymce/core/selection/RangeWalk'
import GetBookmark from 'tinymce/core/bookmark/GetBookmark'
import FormatUtils from 'tinymce/core/fmt/FormatUtils'

const plugin = editor => {
  const createFormatList = function(editor) {
    let count = 0
    const newFormats = []

    const createMenu = function(formats) {
      const menu = []

      if (!formats) {
        return
      }

      Tools.each(formats, function(format) {
        const menuItem: any = {
          text: format.title,
          icon: format.icon,
        }

        if (format.items) {
          menuItem.menu = createMenu(format.items)
        } else {
          const formatName = format.format || 'custom' + count++

          if (!format.format) {
            format.name = formatName
            newFormats.push(format)
          }

          menuItem.format = formatName
          menuItem.cmd = format.cmd
        }

        menu.push(menuItem)
      })

      return menu
    }

    editor.on('init', function() {
      Tools.each(newFormats, function(format) {
        editor.formatter.register(format.name, format)
      })
    })

    return createMenu(editor.settings.style_formats)
  }

  const createListBoxChangeHandler = function(editor, items) {
    return function() {
      const self = this

      editor.on('nodeChange', function(e) {
        const formatter = editor.formatter
        let value = null
        let text = null

        Tools.each(e.parents, function(node) {
          Tools.each(items, function(item) {
            if (formatter.matchNode(node, item.format)) {
              value = item.format
              text = item.text
            }

            if (value) {
              return false
            }
          })

          if (value) {
            return false
          }
        })

        self.value(value)
        if (text) {
          self.text(text)
        }
      })
    }
  }

  const setRangeFormatterApply = formatName => {
    const disableFormatSelector = 'figure,figcaption,div,img'
    const selectionNode = editor.selection.getNode()
    const isCollapsed = editor.selection.isCollapsed()
    const rangeNodes: Array<Node> = []
    const bookmark = GetBookmark.getPersistentBookmark(editor.selection, true)
    RangeWalk.walk(editor.dom, editor.selection.getRng(), function(nodes: Array<Node>) {
      nodes.forEach(node => {
        if (node.nodeName == '#text') {
          node = node.parentNode
        }
        rangeNodes.push(node)
      })
    })

    if (rangeNodes.length > 0) {
      rangeNodes.forEach(node => {
        if (!editor.dom.is(node, disableFormatSelector)) {
          editor.formatter.apply(formatName, null, node)
        }
      })
    } else if (isCollapsed && editor.dom.is(selectionNode, 'b,i,u,s,p,span,h1,h2,h3,h4,h5,h6,blockquote')) {
      editor.formatter.apply(formatName, null, selectionNode)
    }
    editor.selection.moveToBookmark(bookmark)
    FormatUtils.moveStart(editor.dom, editor.selection, editor.selection.getRng())
    editor.focus()
  }

  const registerButtons = function(editor, list) {
    const items = []
    const align = editor.settings.style_formats_align || 'left'

    Tools.each(list, function(block) {
      items.push({
        text: block.text,
        value: block.format,
        icon: block.icon,
        textStyle() {
          return editor.formatter.getCssText(block.format)
        },
      })
    })
    editor.addButton('styleFormat', {
      type: 'listbox',
      text: items[0].text,
      fixedWidth: true,
      tooltip: 'Formats',
      menu: {
        classes: 'style-format',
        items,
        onPostRender(e) {
          e.control.fixed(true)
        },
      },
      onPostRender: createListBoxChangeHandler(editor, list),
      onselect(e) {
        if (e.control) {
          console.log(e.control);
          const formatName = e.control.value()
          setRangeFormatterApply(formatName)
          this.value(formatName)
          this.text(e.control.text())
        }
      },
      onShowMenu() {
        if (align === 'center') {
          this.menu.moveRel(this.getEl(), ['bc-tc', 'tc-bc'])
        }
      },
    })
  }

  registerButtons(editor, createFormatList(editor))

}

export default plugin
