import openFile from '../../lib/openFile'
import VideoUploader from './VideoUploader'
import VideoUploadDialog from './VideoUploadDialog'
import ResizePanel from './ui/ResizePanelAction'

const plugin = editor => {
  let onButtonClicked = () => {}

  if (editor.settings.video) {
    const video = editor.settings.video

    if (video.upload_url || video.upload_handler) {
      try {
        const uploader = new VideoUploader(editor)
        onButtonClicked = () => {
          openFile({ multiple: false, accept: 'video/mp4,video/x-m4v,video/*' }).then(files => {
            editor.undoManager.ignore(() => {
              uploader.upload(files[0])
            })
          })
        }
      } catch (e) {
        console.error(e.message)
        return
      }
    } else if (video.service_name) {
      const dialog = new VideoUploadDialog(editor)
      onButtonClicked = () => dialog.open()
    }

    editor.addButton('video', {
      name: 'Video',
      tooltip: '동영상',
      onclick: () => onButtonClicked(),
    })

    editor.addCommand('KVideoUpload', onButtonClicked)
    editor.addCommand('VideoMobile', onButtonClicked)
  }

  editor.on('PreProcess', e => {
    const $ = editor.dom.$
    const dom = editor.dom

    $('[data-ke-type="video"][data-video-vid]', e.node).each((idx, elm) => {
      const $elm = $(elm)
      const url = $elm.attr('data-video-url')
      const thumbnail = $elm.attr('data-video-thumbnail')
      const vid = $elm.attr('data-video-vid')
      const uploaderHost = $elm.attr('data-video-uploader-host')
      const videoHost = $elm.attr('data-video-host')

      if (!videoHost && !vid) {
        $elm.remove()
        throw new Error('동영상 업로드가 완료된 후 시도해 주세요.')
      }

      if (url) {
        $elm[0].outerHTML = `<iframe
        width="630" height="390"
        frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen
        src='${url}'
        data-ke-type="video"
        data-video-url='${url}'
        data-video-thumbnail='${thumbnail}'
        data-video-vid='${vid}'
        data-video-uploader-host='${uploaderHost}'></iframe>`
      } else {
        $elm[0].outerHTML = `<figure data-ke-type="video" data-video-thumbnail='${thumbnail}' data-video-vid='${vid}' data-video-uploader-host='${uploaderHost}'><div class="video_wrap"><img src="${thumbnail}" /></div></figure>`
      }
    })

    $('figure[data-ke-type="video"][data-video-host]', e.node).each((idx, elm) => {
      const $elm = $(elm)
      const host = $elm.attr('data-video-host')
      const dataVideoUrl = $elm.attr('data-video-url')
      const caption = $elm.find('figcaption').text()
      const $imageElem = $elm.find('img')

      const figureAttr = {
        'data-ke-type': 'video',
        'data-ke-style': $elm.attr('data-ke-style'),
        'data-video-host': host,
        'data-video-url': $elm.attr('data-video-url'),
        'data-video-thumbnail': $elm.attr('data-video-thumbnail'),
        'data-original-url': $elm.attr('data-original-url'),
      }

      const playerAttr = {
        type: 'text/html',
        allow: 'autoplay; encrypted-media',
        allowfullscreen: true,
        webkitallowfullscreen: true,
        mozallowfullscreen: true,
        frameborder: '0',
        width: Number($imageElem.attr('width')) || 710,
        height: Number($imageElem.attr('height')) || 418,
        src: dataVideoUrl,
      }

      if (host == 'youtube') {
        playerAttr.src = dataVideoUrl.replace('watch?v=', 'embed/')
      } else if (host == 'kakaotv') {
        playerAttr.src = `https://play-tv.kakao.com/embed/player/cliplink/${
          elm.getAttribute('data-video-url').split('cliplink/')[1]
        }?service=kakao_tv`
      } else if (host == 'vimeo') {
        playerAttr.src = `https://player.vimeo.com/video/${elm.getAttribute('data-video-url').split('vimeo.com/')[1]}`
      }

      const figure = dom.create('figure', figureAttr)
      dom.add(figure, dom.create('iframe', playerAttr))
      dom.add(figure, dom.create('figcaption', {}, caption))
      editor.dom.replace(figure, elm, false)
    })
  })

  editor.on('SetContent', e => {
    const $ = editor.dom.$
    $('figure[data-ke-type="video"]', e.node).each((idx, elm) => {
      if (elm.className == 'uploading') {
        return
      }

      const $elm = $(elm)
      const url = $elm.attr('data-video-url')
      const thumbnail = $elm.attr('data-video-thumbnail')
      const vid = $elm.attr('data-video-vid') || ''
      const uploaderHost = $elm.attr('data-video-host') || ''
      const caption = $elm.find('figcaption').text()
      const $player = $elm.find('iframe')
      if (!url || !thumbnail || $player.length == 0) {
        return
      }

      let vidAttr = ''
      if (vid) {
        vidAttr = `data-video-vid='${vid}'`
      }

      elm.outerHTML = `<figure data-ke-type="video" data-video-url='${url}' data-video-thumbnail='${thumbnail}' ${vidAttr} data-video-host='${uploaderHost}' contentEditable='false'>
        <div class='video_content'>
          <img src='${thumbnail}' width='${$player.attr('width')}' height='${$player.attr('height')}'>
        </div>
        <figcaption>${caption}</figcaption>
      </figure>`
    })

    $('iframe[data-ke-type="video"]', e.node).each((idx, elm) => {
      if (elm.className == 'uploading') {
        return
      }
      const $elm = $(elm)
      const url = $elm.attr('data-video-url')
      const thumbnail = $elm.attr('data-video-thumbnail')
      const vid = $elm.attr('data-video-vid')
      const uploaderHost = $elm.attr('data-video-uploader-host')
      const videoHost = $elm.attr('data-video-host')
      const itemStyle = $elm.attr('data-ke-style')
      const width = $elm.attr('width')
      const height = $elm.attr('height')

      if (videoHost) {
        const $wrapParagraph = $(elm.parentNode)
        if ($wrapParagraph[0].nodeName == 'P') {
          $wrapParagraph.before($elm)
          if ($wrapParagraph[0].childElementCount == 0) {
            $wrapParagraph.remove()
          }
        }
        $elm[0].outerHTML = `<figure data-ke-type="video" data-ke-style="${itemStyle}" data-video-url="${url}" data-video-thumbnail="${thumbnail}" data-video-host="${videoHost}" contentEditable="false">
        <img width="${width}" height="${height}" src='${thumbnail}'>
        <span><br></span>
      </figure>`
      } else {
        if (!vid || !url || !thumbnail) {
          return
        }
        $elm[0].outerHTML = `<figure data-ke-type="video" data-ke-style="${itemStyle}" data-video-url="${url}" data-video-thumbnail="${thumbnail}" data-video-vid="${vid}" data-video-upload-host="${uploaderHost}" contentEditable="false">
      <div class="video_wrap">
        <img src='${thumbnail}'>
        <span><br></span>
      </div>
      </figure>`
      }
    })
  })

  const postrender = function(e) {
    const control = this

    const toggleActive = () => {
      const $figureNode = editor.$(editor.selection.getNode()).closest('figure')
      const keStyle = $figureNode.attr('data-ke-style')
      control.active(control.settings.activestate == keStyle)
    }

    toggleActive()

    editor.on('nodeChange', e => {
      toggleActive()
    })
  }

  editor.addButton('videoAlignLeft', {
    title: 'Align left',
    cmd: 'kImageAlignLeft',
    icon: 'imagealignleft',
    activestate: 'alignLeft',
    onpostrender: postrender,
  })

  editor.addButton('videoAlignCenter', {
    title: 'Align center',
    cmd: 'kImageAlignCenter',
    icon: 'imagealigncenter',
    activestate: 'alignCenter',
    onpostrender: postrender,
  })

  editor.addButton('videoAlignRight', {
    title: 'Align right',
    cmd: 'kImageAlignRight',
    icon: 'imagealignright',
    activestate: 'alignRight',
    onpostrender: postrender,
  })

  const panel = new ResizePanel(editor, 'videoResizeWidthInput')

  editor.addButton('videoResize', {
    title: 'Resize Image',
    icon: 'resizeimage',
    type: 'panelbutton',
    tooltip: 'Resize Image',
    panel: {
      classes: 'kImageResize tinymce',
      role: 'dialog',
      fixed: false,
      onPostRender() {
        panel.init(this)
      },
      items: [
        { type: 'label', classes: 'widthLabel', text: 'W' },
        {
          type: 'textbox',
          id: 'videoResizeWidthInput',
          classes: 'kImageWidthInput',
          size: 3,
          maxLength: 3,
          label: 'Width',
          name: 'width',
          onsubmit: function() {
            panel.updateImageWidth(this.value())
            this.parent().hide()
          },
        },
        {
          type: 'button',
          classes: 'kImageMaxSize',
          title: 'Resize image Max size',
          icon: 'reiszeimagemax',
          tooltip: 'Max size',
          onclick: panel.updateMaxSize.bind(panel),
        },
      ],
    },
    onshow(e) {
      e.control.fixed(false)
      panel.show()
    },
    onhide() {
      editor.focus()
    },
  })

  const predicate = element => {
    return editor.dom.is(element, 'figure[data-ke-type="video"]')
  }

  editor.addContextToolbar(predicate, 'videoResize | videoAlignLeft videoAlignCenter videoAlignRight')
}

export default plugin
