import { Option } from '@ephox/katamari'
const DOMUtils = tinymce.dom.DOMUtils
const Rect = tinymce.geom.Rect
const Factory = tinymce.ui.Factory
const Delay = tinymce.util.Delay
const Tools = tinymce.util.Tools
const UiContainer = tinymce.ui.UiContainer
const Env = tinymce.Env

const getInlineToolbarPositionHandler = (editor) => {
  return editor.getParam('inline_toolbar_position_handler');
};
const createToolbar = (editor, items, size) => {
  const toolbarItems = [];
  let buttonGroup;
  if (!items) {
    return;
  }
  Tools.each(items.split(/[ ,]/), function (item) {
    let itemName;
    const bindSelectorChanged = function () {
      const selection = editor.selection;
      if (item.settings.stateSelector) {
        selection.selectorChanged(item.settings.stateSelector, function (state) {
          item.active(state);
        }, true);
      }
      if (item.settings.disabledStateSelector) {
        selection.selectorChanged(item.settings.disabledStateSelector, function (state) {
          item.disabled(state);
        });
      }
    };
    if (item === '|') {
      buttonGroup = null;
    } else {
      if (!buttonGroup) {
        buttonGroup = {
          type: 'buttongroup',
          items: []
        };
        toolbarItems.push(buttonGroup);
      }
      if (editor.buttons[item]) {
        itemName = item;
        item = editor.buttons[itemName];
        if (typeof item === 'function') {
          item = item();
        }
        item.type = item.type || 'button';
        item.size = size;
        item = Factory.create(item);
        buttonGroup.items.push(item);
        if (editor.initialized) {
          bindSelectorChanged();
        } else {
          editor.on('init', bindSelectorChanged);
        }
      }
    }
  });
  return {
    type: 'toolbar',
    layout: 'flow',
    items: toolbarItems
  };
};

const getUiContainerDelta = () => {
  const uiContainer = Env.container;
  if (uiContainer && DOMUtils.DOM.getStyle(uiContainer, 'position', true) !== 'static') {
    const containerPos = DOMUtils.DOM.getPos(uiContainer);
    const dx = uiContainer.scrollLeft - containerPos.x;
    const dy = uiContainer.scrollTop - containerPos.y;
    return Option.some({
      x: dx,
      y: dy
    });
  } else {
    return Option.none()
  }
};

const setUiContainer = (editor, ctrl) => {
  const uiContainer = DOMUtils.DOM.select(editor.settings.ui_container)[0];
  ctrl.getRoot().uiContainer = uiContainer;
};

const getUiContainer = (ctrl) => ctrl ? ctrl.getRoot().uiContainer : null;

const DOM = DOMUtils.DOM;

const toClientRect = function (geomRect) {
  return {
    left: geomRect.x,
    top: geomRect.y,
    width: geomRect.w,
    height: geomRect.h,
    right: geomRect.x + geomRect.w,
    bottom: geomRect.y + geomRect.h
  };
};

const hideAllFloatingPanels = function (editor) {
  Tools.each(editor.contextToolbars, function (toolbar) {
    if (toolbar.panel) {
      toolbar.panel.hide();
    }
  });
};

const movePanelTo = function (panel, pos) {
  panel.moveTo(pos.left, pos.top);
};

const togglePositionClass = function (panel, relPos, predicate) {
  relPos = relPos ? relPos.substr(0, 2) : '';

  Tools.each({
    t: 'down',
    b: 'up'
  }, function (cls, pos) {
    panel.classes.toggle('arrow-' + cls, predicate(pos, relPos.substr(0, 1)));
  });

  Tools.each({
    l: 'left',
    r: 'right'
  }, function (cls, pos) {
    panel.classes.toggle('arrow-' + cls, predicate(pos, relPos.substr(1, 1)));
  });
};

const userConstrain = function (handler, x, y, elementRect, contentAreaRect, panelRect) {
  panelRect = toClientRect({ x, y, w: panelRect.w, h: panelRect.h });

  if (handler) {
    panelRect = handler({
      elementRect: toClientRect(elementRect),
      contentAreaRect: toClientRect(contentAreaRect),
      panelRect
    });
  }

  return panelRect;
};

const addContextualToolbars = function (editor) {
  let scrollContainer;

  const getContextToolbars = function () {
    return editor.contextToolbars || [];
  };

  const getElementRect = function (elm) {
    let pos, targetRect, root;

    pos = DOM.getPos(editor.getContentAreaContainer());
    targetRect = editor.dom.getRect(elm);
    root = editor.dom.getRoot();

    // Adjust targetPos for scrolling in the editor
    if (root.nodeName === 'BODY') {
      targetRect.x -= root.ownerDocument.documentElement.scrollLeft || root.scrollLeft;
      targetRect.y -= root.ownerDocument.documentElement.scrollTop || root.scrollTop;
    }

    targetRect.x += pos.x;
    targetRect.y += pos.y;

    return targetRect;
  };

  /*
  Rect.findBestRelativePosition 의 
  */

  const findBestRelativePosition = (rect, targetRect, constrainRect, rels) => {
    let pos, i;
  
    for (i = 0; i < rels.length; i++) {
      pos = Rect.relativePosition(rect, targetRect, rels[i]);

      if (rels[i].split('-')[0] == 'cc') {
        if (pos.x >= constrainRect.x && pos.x + pos.w <= constrainRect.w + constrainRect.x &&
          targetRect.y + targetRect.h >= constrainRect.y + pos.h * 2 && targetRect.y + pos.h * 2 <= constrainRect.h + constrainRect.y &&
          pos.h * 1.5 <= targetRect.h && pos.w * 1.5 <= targetRect.w) {
          return rels[i];
        } else {
          null
        }
      } else {
        if (pos.x >= constrainRect.x && pos.x + pos.w <= constrainRect.w + constrainRect.x &&
          pos.y >= constrainRect.y && pos.y + pos.h <= constrainRect.h + constrainRect.y) {
          return rels[i];
        } else {
          null
        }
      }
    }
    return null;
  }

  const reposition = function (match, shouldShow) {
    let relPos, panelRect, elementRect, contentAreaRect, panel, relRect, testPositions, smallElementWidthThreshold;
    const handler = getInlineToolbarPositionHandler(editor);

    if (editor.removed) {
      return;
    }

    if (!match || !match.toolbar.panel) {
      hideAllFloatingPanels(editor);
      return;
    }

    testPositions = [
      'tc-bc', 'cc-cc', 'bc-tc',
      // 'tl-bl', 'bl-tl',
      // 'tr-br', 'br-tr'
    ];

    if (match && match.element.nodeName == 'TABLE') {
      testPositions = ['tc-bc','bc-tc']
    }

    if (match && (match.element.nodeName == 'IMG' || match.element.nodeName == 'FIGURE')) {
      testPositions = ['cc-cc']
    }

    panel = match.toolbar.panel;

    // Only show the panel on some events not for example nodeChange since that fires when context menu is opened
    if (shouldShow) {
      panel.show();
    }

    elementRect = getElementRect(match.element);
    panelRect = DOM.getRect(panel.getEl());
    contentAreaRect = DOM.getRect(editor.getContentAreaContainer() || editor.getBody());

    const delta = getUiContainerDelta(panel).getOr({ x: 0, y: 0 });
    elementRect.x += delta.x;
    elementRect.y += delta.y;
    panelRect.x += delta.x;
    panelRect.y += delta.y;
    contentAreaRect.x += delta.x;
    contentAreaRect.y += delta.y;

    smallElementWidthThreshold = 25;

    if (DOM.getStyle(match.element, 'display', true) !== 'inline') {
      // We need to use these instead of the rect values since the style
      // size properites might not be the same as the real size for a table if it has a caption
      const clientRect = match.element.getBoundingClientRect();
      elementRect.w = clientRect.width;
      elementRect.h = clientRect.height;
    }

    if (!editor.inline) {
      contentAreaRect.w = editor.getDoc().documentElement.offsetWidth;
    }

    // Inflate the elementRect so it doesn't get placed above resize handles
    if (editor.selection.controlSelection.isResizable(match.element) && elementRect.w < smallElementWidthThreshold) {
      elementRect = Rect.inflate(elementRect, 0, 8);
    }

    relPos = findBestRelativePosition(panelRect, elementRect, contentAreaRect, testPositions);
    elementRect = Rect.clamp(elementRect, contentAreaRect);

    if (relPos) {
      relRect = Rect.relativePosition(panelRect, elementRect, relPos);
      movePanelTo(panel, userConstrain(handler, relRect.x, relRect.y, elementRect, contentAreaRect, panelRect));
    } else {
      // 화면 상황에 맞는 추가 toolbar의 표시를 제한
      // // Allow overflow below the editor to avoid placing toolbars ontop of tables
      // contentAreaRect.h += panelRect.h;

      // elementRect = Rect.intersect(contentAreaRect, elementRect);
      // if (elementRect) {
      //   relPos = Rect.findBestRelativePosition(panelRect, elementRect, contentAreaRect, [
      //     'tc-bc', 'bc-tc', 'bl-tl', 'br-tr'
      //   ]);

      //   if (relPos) {
      //     relRect = Rect.relativePosition(panelRect, elementRect, relPos);
      //     movePanelTo(panel, userConstrain(handler, relRect.x, relRect.y, elementRect, contentAreaRect, panelRect));
      //   } else {
      //     movePanelTo(panel, userConstrain(handler, elementRect.x, elementRect.y, elementRect, contentAreaRect, panelRect));
      //   }
      // } else {
        panel.hide();
      // }
    }

    togglePositionClass(panel, relPos, function (pos1, pos2) {
      return pos1 === pos2;
    });

    // drawRect(contentAreaRect, 'blue');
    // drawRect(elementRect, 'red');
    // drawRect(panelRect, 'green');
  };

  const repositionHandler = function (show) {
    return function () {
      const execute = function () {
        if (editor.selection) {
          reposition(findFrontMostMatch(editor.selection.getNode()), show)
        }
      };

      Delay.requestAnimationFrame(execute);
    };
  };

  const bindScrollEvent = function (panel) {
    if (!scrollContainer) {
      const reposition = repositionHandler(true);
      const uiContainer = getUiContainer(panel);

      scrollContainer = editor.selection.getScrollContainer() || editor.getWin();
      DOM.bind(scrollContainer, 'scroll', reposition);
      DOM.bind(uiContainer, 'scroll', reposition);

      editor.on('remove', function () {
        DOM.unbind(scrollContainer, 'scroll', reposition);
        DOM.unbind(uiContainer, 'scroll', reposition);
      });
    }
  };

  const showContextToolbar = function (match) {
    let panel;

    if (match.toolbar.panel) {
      match.toolbar.panel.show();
      reposition(match);
      return;
    }

    panel = Factory.create({
      type: 'floatpanel',
      role: 'dialog',
      classes: 'tinymce tinymce-inline arrow',
      ariaLabel: 'Inline toolbar',
      layout: 'flex',
      direction: 'column',
      align: 'stretch',
      autohide: false,
      autofix: true,
      fixed: true,
      border: 1,
      items: createToolbar(editor, match.toolbar.items),
      oncancel () {
        editor.focus();
      }
    });

    setUiContainer(editor, panel);
    bindScrollEvent(panel);

    match.toolbar.panel = panel;
    panel.renderTo().reflow();
    reposition(match);
  };

  const hideAllContextToolbars = function () {
    Tools.each(getContextToolbars(), function (toolbar) {
      if (toolbar.panel) {
        toolbar.panel.hide();
      }
    });
  };

  const findFrontMostMatch = function (targetElm) {
    let i, y, parentsAndSelf;
    const toolbars = getContextToolbars();

    parentsAndSelf = editor.$(targetElm).parents().add(targetElm);
    for (i = parentsAndSelf.length - 1; i >= 0; i--) {
      for (y = toolbars.length - 1; y >= 0; y--) {
        if (toolbars[y].predicate(parentsAndSelf[i])) {
          return {
            toolbar: toolbars[y],
            element: parentsAndSelf[i]
          };
        }
      }
    }

    return null;
  };

  editor.on('click keyup setContent ObjectResized', function (e) {
    // Only act on partial inserts
    if (e.type === 'setcontent' && !e.selection) {
      return;
    }

    // Needs to be delayed to avoid Chrome img focus out bug
    Delay.setEditorTimeout(editor, function () {
      let match;

      match = findFrontMostMatch(editor.selection.getNode());
      if (match) {
        hideAllContextToolbars();
        showContextToolbar(match);
      } else {
        hideAllContextToolbars();
      }
    });
  });

  editor.on('blur hide contextmenu', hideAllContextToolbars);

  editor.on('ObjectResizeStart', function () {
    const match = findFrontMostMatch(editor.selection.getNode());

    if (match && match.toolbar.panel) {
      match.toolbar.panel.hide();
    }
  });

  editor.on('ResizeEditor ResizeWindow', repositionHandler(true));
  editor.on('nodeChange', repositionHandler(false));

  editor.on('remove', function () {
    Tools.each(getContextToolbars(), function (toolbar) {
      if (toolbar.panel) {
        toolbar.panel.remove();
      }
    });

    editor.contextToolbars = {};
  });

  editor.shortcuts.add('ctrl+shift+e > ctrl+shift+p', '', function () {
    const match = findFrontMostMatch(editor.selection.getNode());
    if (match && match.toolbar.panel) {
      match.toolbar.panel.items()[0].focus();
    }
  })
}

export default { addContextualToolbars }
