import { AlloyTriggers, Attachment, Swapping } from '@ephox/alloy'
import { Cell, Fun } from '@ephox/katamari'
import { PlatformDetection } from '@ephox/sand'
import { Element, Focus, Insert, Node, Platform } from '@ephox/sugar'
import DOMUtils from 'tinymce/core/api/dom/DOMUtils'
// import ThemeManager from 'tinymce/core/api/ThemeManager';

import TinyCodeDupe from './alien/TinyCodeDupe'
import * as Settings from './api/Settings'
import TinyChannels from './channels/TinyChannels'
import Features from './features/Features'
import Styles from './style/Styles'
import Orientation from './touch/view/Orientation'
import AndroidRealm from './ui/AndroidRealm'
import Buttons from './ui/Buttons'
import IosRealm from './ui/IosRealm'
import CssUrls from './util/CssUrls'
import FormatChangers from './util/FormatChangers'
import SkinLoaded from './util/SkinLoaded'
import mobilePluginInit from 'src/lib/MobilePluginManager/MobilePluginManager'

/// not to be confused with editor mode
const READING = Fun.constant('toReading') /// 'hide the keyboard'
const EDITING = Fun.constant('toEditing') /// 'show the keyboard'

declare global {
  interface Window {
    tinymce: any
  }
}
const tinymce = window.tinymce || {}
const ThemeManager = tinymce.util.Tools.resolve('tinymce.ThemeManager')
mobilePluginInit()

ThemeManager.add('mobileClassic', function(editor) {
  const renderUI = function(args) {
    const cssUrls = CssUrls.derive(editor)

    if (Settings.isSkinDisabled(editor) === false) {
      editor.contentCSS.push(cssUrls.content)
      DOMUtils.DOM.styleSheetLoader.load(cssUrls.ui, SkinLoaded.fireSkinLoaded(editor))
    } else {
      SkinLoaded.fireSkinLoaded(editor)()
    }

    const doScrollIntoView = function() {
      editor.fire('scrollIntoView')
    }

    const wrapper = Element.fromTag('div')
    const realm = PlatformDetection.detect().os.isAndroid()
      ? AndroidRealm(doScrollIntoView)
      : IosRealm(doScrollIntoView)
    const original = Element.fromDom(args.targetNode)
    Insert.after(original, wrapper)
    Attachment.attachSystem(wrapper, realm.system())

    const findFocusIn = function(elem) {
      return Focus.search(elem).bind(function(focused) {
        return realm
          .system()
          .getByDom(focused)
          .toOption()
      })
    }
    const outerWindow = args.targetNode.ownerDocument.defaultView
    const orientation = Orientation.onChange(outerWindow, {
      onChange() {
        const alloy = realm.system()
        alloy.broadcastOn([TinyChannels.orientationChanged()], { width: Orientation.getActualWidth(outerWindow) })
      },
      onReady: Fun.noop,
    })

    const setReadOnly = function(mainGroups, firstGroups, thirdGroups, ro) {
      if (ro === false) {
        editor.selection.collapse()
      }
      const toolbars = configureToolbar(mainGroups, firstGroups, thirdGroups)
      realm.setToolbarGroups({
        main: toolbars.main,
        first: toolbars.first,
        third: toolbars.third,
      })

      editor.setMode(ro === true ? 'readonly' : 'design')
      editor.fire(ro === true ? READING() : EDITING())
      realm.updateMode(ro)
    }

    const configureToolbar = function(mainGroups, firstGroups, thirdGroups) {
      const toolbars = {
        main: mainGroups.get(),
        first: firstGroups.get(),
        third: thirdGroups.get(),
      }

      if (Settings.readOnlyOnInit(editor)) {
      }

      return toolbars
    }

    const bindHandler = function(label, handler) {
      editor.on(label, handler)
      return {
        unbind() {
          editor.off(label)
        },
      }
    }

    editor.on('init', function() {
      realm.init({
        editor: {
          getFrame() {
            return Element.fromDom(editor.contentAreaContainer.querySelector('iframe'))
          },

          onDomChanged() {
            return {
              unbind: Fun.noop,
            }
          },

          onToReading(handler) {
            return bindHandler(READING(), handler)
          },

          onToEditing(handler) {
            return bindHandler(EDITING(), handler)
          },

          onScrollToCursor(handler) {
            editor.on('scrollIntoView', function(tinyEvent) {
              handler(tinyEvent)
            })

            const unbind = function() {
              editor.off('scrollIntoView')
              orientation.destroy()
            }

            return {
              unbind,
            }
          },

          onTouchToolstrip() {
            hideDropup()
            realm.system().broadcastOn([TinyChannels.menuTouched()], {})
          },

          onTouchContent() {
            const toolbar = Element.fromDom(editor.editorContainer.querySelector('.' + Styles.resolve('toolbar')))
            // If something in the toolbar had focus, fire an execute on it (execute on tap away)
            // Perhaps it will be clearer later what is a better way of doing this.
            findFocusIn(toolbar).each(AlloyTriggers.emitExecute)
            realm.restoreToolbar()
            hideDropup()
            realm.system().broadcastOn([TinyChannels.menuTouched()], {})
          },

          onTapContent(evt) {
            const target = evt.target()
            // If the user has tapped (touchstart, touchend without movement) on an image, select it.
            if (Node.name(target) === 'img') {
              editor.selection.select(target.dom())
              // Prevent the default behaviour from firing so that the image stays selected
              evt.kill()
            } else if (Node.name(target) === 'a') {
              const component = realm.system().getByDom(Element.fromDom(editor.editorContainer))
              component.each(function(container) {
                /// view mode
                if (Swapping.isAlpha(container)) {
                  TinyCodeDupe.openLink(target.dom())
                }
              })
            }
          },
        },
        container: Element.fromDom(editor.editorContainer),
        socket: Element.fromDom(editor.contentAreaContainer),
        toolstrip: Element.fromDom(editor.editorContainer.querySelector('.' + Styles.resolve('toolstrip'))),
        toolbar: Element.fromDom(editor.editorContainer.querySelector('.' + Styles.resolve('toolbar'))),
        dropup: realm.dropup(),
        alloy: realm.system(),
        translate: Fun.noop,

        setReadOnly(ro) {
          setReadOnly(mainGroups, firstGroups, thirdGroups, ro)
        },

        readOnlyOnInit() {
          return Settings.readOnlyOnInit(editor)
        },
      })

      const hideDropup = function() {
        realm.dropup().disappear(function() {
          realm.system().broadcastOn([TinyChannels.dropupDismissed()], {})
        })
      }

      // const backToMaskGroup = {
      //   label: 'The first group',
      //   scrollable: false,
      //   items: [
      //     // Buttons.forToolbar('back', function (/* btn */) {
      //     //   editor.selection.collapse();
      //     //   realm.exit();
      //     // }, { })
      //   ]
      // };

      // const backToReadOnlyGroup = {
      //   label: 'Back to read only',
      //   scrollable: false,
      //   items: [
      //     Buttons.forToolbar('readonly-back', function (/* btn */) {
      //       setReadOnly(dynamicGroup, readOnlyGroups, mainGroups, firstGroups, thirdGroups, true);
      //     }, {})
      //   ]
      // };

      // const readOnlyGroup = {
      //   label: 'The read only mode group',
      //   scrollable: true,
      //   items: []
      // };

      const features = Features.setup(realm, editor)
      const items = []
      features.forEach(seperatedFeatures => {
        items.push(Features.detect(editor.settings, seperatedFeatures))
      })

      const mainLeftGroup = {
        label: 'the action header group',
        scrollabe: false,
        items: items[1],
      }
      const mainRightGroup = {
        label: 'the test2 group',
        scrollable: false,
        items: items[2],
      }

      const extraGroup = {
        label: 'The extra group',
        scrollable: false,
        items: [],
      }

      const topGroup = {
        label: 'the top group',
        scrollabe: false,
        items: items[0],
      }

      const thirdGroup = {
        label: 'the header group',
        scrollable: false,
        items: items[3] || [],
      }

      const fourthGroup = {
        label: 'the font group',
        scrollable: false,
        items: [],
      }

      const mainGroups = Cell([mainLeftGroup, mainRightGroup])
      // const readOnlyGroups = Cell([ readOnlyGroup, extraGroup ]);
      // const dynamicGroup = Cell({
      //   backToMask: [ backToMaskGroup ],
      //   backToReadOnly: [ backToReadOnlyGroup ]
      // });

      const firstGroups = Cell([topGroup, extraGroup])
      const thirdGroups = Cell([thirdGroup, extraGroup])
      // Investigate ways to keep in sync with the ui
      FormatChangers.init(realm, editor)
    })

    editor.addCommand('exitMobileMode', () => {
      editor.selection.collapse()
      realm.exit()
    })

    editor.addCommand('mobilePublish', () => {
      const settings = editor.settings
      if (settings && settings.publishCallback) {
        settings.publishCallback()
      }
      editor.selection.collapse()
      realm.exit()
    })

    return {
      iframeContainer: realm
        .socket()
        .element()
        .dom(),
      editorContainer: realm.element().dom(),
    }
  }

  /**
   * 모바일 모드 일때 class 추가
   * content body에 mobile-content-body class 추가
   * cloud.lee 2018.08.09
   */

  editor.on('init', function() {
    editor.$(editor.getBody()).addClass('mobile-content-body')
    if (PlatformDetection.detect().os.isiOS()) {
      editor.$(editor.getBody()).css('width', 'calc(100vw - 0.1px)')
    }
  })

  return {
    getNotificationManagerImpl() {
      return {
        open: Fun.identity,
        close: Fun.noop,
        reposition: Fun.noop,
        getArgs: Fun.identity,
      }
    },
    renderUI,
  }
})

export default function() {}
