import { Replacing, ComponentApi, Gui } from '@ephox/alloy';
import { Fun, Singleton } from '@ephox/katamari';

import IosWebapp, { MobileWebApp } from '../api/IosWebapp';
import Styles from '../style/Styles';
import ScrollingToolbar from '../toolbar/ScrollingToolbar';
import CommonRealm from './CommonRealm';
import * as Dropup from './Dropup';
import OuterContainer from './OuterContainer';
import { SugarElement } from '../alien/TypeDefinitions';

export interface MobileRealm {
  system(): Gui.GuiSystem;
  element(): SugarElement;
  init(spec): void;
  exit(): void;
  setToolbarGroups(rawGroups): void;
  setContextToolbar(rawGroups): void;
  focusToolbar(): void;
  restoreToolbar(): void;
  closeFourthToolbar(): void;
  updateMode(readOnly: boolean): void;
  socket(): ComponentApi.AlloyComponent;
  dropup(): Dropup.DropUp;
}

export default function (scrollIntoView: () => void) {
  const alloy = OuterContainer({
    classes: [ Styles.resolve('ios-container') ]
  }) as Gui.GuiSystem;

  const toolbarCatalogue = {
    first: ScrollingToolbar(),
    second: ScrollingToolbar(),
    third: ScrollingToolbar(),
    fourth: ScrollingToolbar(),
  };

  const webapp = Singleton.api<MobileWebApp>();

  const switchToEdit = CommonRealm.makeEditSwitch(webapp);

  const socket = CommonRealm.makeSocket() as ComponentApi.AlloyComponent;

  const dropup = Dropup.build(function () {
    webapp.run(function (w) {
      w.refreshStructure();
    });
  }, scrollIntoView);

  for(const toolbar in toolbarCatalogue) {
    alloy.add(toolbarCatalogue[toolbar].wrapper());
  }

  alloy.add(socket);
  alloy.add(dropup.component());

  toolbarCatalogue.third.wrapper().element().dom().style.display = "none";
  toolbarCatalogue.fourth.wrapper().element().dom().style.display = "none";

  const setToolbarGroups = function (rawGroups) {
    const mainGroups = toolbarCatalogue.second.createGroups(rawGroups.main);
    const firstGroups = toolbarCatalogue.first.createGroups(rawGroups.first);
    const thirdGroups = toolbarCatalogue.third.createGroups(rawGroups.third);
    toolbarCatalogue.second.setGroups(mainGroups);
    toolbarCatalogue.first.setGroups(firstGroups);
    toolbarCatalogue.third.setGroups(thirdGroups);
  };

  const setContextToolbar = function (rawGroups, target) {
    if(target === 3) {
      toolbarCatalogue.third.wrapper().element().dom().style.display = "flex";
      const groups = toolbarCatalogue.third.createGroups(rawGroups);
      toolbarCatalogue.third.setContextToolbar(groups);
    }
    else if(target === 4) {
      toolbarCatalogue.fourth.wrapper().element().dom().style.display = "flex";
      const groups = toolbarCatalogue.fourth.createGroups(rawGroups);
      toolbarCatalogue.fourth.setContextToolbar(groups);
    }
    // forceScroll();
  };

  const focusToolbar = function () {
    toolbarCatalogue.second.focus();
  };

  const restoreToolbar = function () {
    toolbarCatalogue.third.wrapper().element().dom().style.display = "none";
    toolbarCatalogue.fourth.wrapper().element().dom().style.display = "none";
    toolbarCatalogue.third.restoreToolbar();
    toolbarCatalogue.fourth.restoreToolbar();
    // forceScroll();
  };
  
  const closeFourthToolbar = function () {
    const fourth = toolbarCatalogue.fourth;
    if(fourth.wrapper().element().dom().style.display === "none") return;

    fourth.restoreToolbar();
    fourth.wrapper().element().dom().style.display = "none";
    // forceScroll();
  }

  // const forceScroll = function () {
    // const isThirdOff = toolbarCatalogue.third.wrapper().element().dom().style.display === "none";
    // const isFourthOff = toolbarCatalogue.fourth.wrapper().element().dom().style.display === "none";

    // let level;
    // if (isThirdOff) level = 2;
    // else if (!isThirdOff && isFourthOff) level = 3;
    // else level = 4;

    // socket.element().dom().style.top = `${level * 45}px`;
  // };

  const init = function (spec) {
    webapp.set(
      IosWebapp.produce(spec)
    );
  };

  const exit = function () {
    webapp.run(function (w) {
      Replacing.remove(socket, switchToEdit);
      w.exit();
    });
  };

  const updateMode = function (readOnly) {
    CommonRealm.updateMode(socket, switchToEdit, readOnly, alloy.root());
  };

  return {
    system: Fun.constant(alloy),
    element: alloy.element as () => SugarElement,
    init,
    exit,
    setToolbarGroups,
    setContextToolbar,
    focusToolbar,
    restoreToolbar,
    closeFourthToolbar,
    updateMode,
    socket: Fun.constant(socket),
    dropup: Fun.constant(dropup)
  } as MobileRealm;
}