import { Assertions, Pipeline } from '@ephox/agar';
import { LegacyUnit, TinyLoader } from '@ephox/mcagar';
import Theme from 'src/themes/classic/main/ts/Theme';
import { UnitTest } from '@ephox/bedrock';
import plugin from '../../../src/plugins/colorPalette/src/plugin';

UnitTest.asynctest('browser.ColorPaletteTest', function () {
  const success = arguments[arguments.length - 2];
  const failure = arguments[arguments.length - 1];
  const suite = LegacyUnit.createSuite();

  Theme();

  window.tinymce.PluginManager.add('colorPalette', plugin);

  const getContent = function (editor) {
    return editor.getContent().toLowerCase().replace(/[\r]+/g, '');
  };

  suite.test('색 적용 테스트', function (editor) {
    const doc = editor.dom.win.document;

    editor.focus();
    editor.getBody().innerHTML = '<p>color no color</p>';

    const rng = editor.dom.createRng();
    rng.setStart(editor.dom.select('p')[0].firstChild, 0);
    rng.setEnd(editor.dom.select('p')[0].firstChild, 5);
    editor.selection.setRng(rng);

    doc.getElementById('mceu_0-button').click();
    const firstButton = doc.getElementsByClassName('colorPalette-preset-td')[0];
    const correctColor = firstButton.getAttribute('data-mce-color').toLowerCase();
    firstButton.click();

    LegacyUnit.equal(
      editor.getContent(),
      `<p><span style="color: ${correctColor};">color</span> no color</p>`,
    );
  });

  suite.test('현재 노드 색에 따른 체크표시 테스트', function (editor) {
    const doc = editor.dom.win.document;
    doc.getElementById('mceu_0-button').click();
    const colorButtons = doc.getElementsByClassName('colorPalette-preset-td');
    const firstButton = colorButtons[0];
    const blackButton = <HTMLInputElement> Array.from(colorButtons).filter(btn => (<HTMLInputElement> btn).getAttribute('data-mce-color') === editor.settings.colorPalette.default_color)[0];
    const correctColor = firstButton.getAttribute('data-mce-color');
    const colorNode = editor.dom.select('span')[0];
    editor.selection.select(colorNode);

    LegacyUnit.equal(
      firstButton.className.includes('colorPalette-selected'),
      true,
      '현재 선택된 색상이므로 체크 표시가 있어야 한다'
    );
    LegacyUnit.equal(
      firstButton.nextSibling.className.includes('colorPalette-selected'),
      false,
      '현재 선택된 색상이 아니므로 체크 표시가 없어야 한다'
    );

    doc.getElementById('mceu_0-button').click();
    editor.selection.select(colorNode.nextSibling);
    doc.getElementById('mceu_0-button').click();

    LegacyUnit.equal(
      blackButton.className.includes('colorPalette-selected'),
      true,
      '현재 선택된 색상이므로 체크 표시가 있어야 한다 (기본색)'
    );
    LegacyUnit.equal(
      firstButton.className.includes('colorPalette-selected'),
      false,
      '현재 선택된 색상이 아니므로 체크 표시가 없어야 한다 (기본색)'
    );
  });

  suite.test('여러 요소에 걸친 색 삽입 테스트', function (editor) {
    const doc = editor.dom.win.document;
    editor.focus();
    editor.getBody().innerHTML = `
      <h2>우리는 좋은 글이 가지는 힘을 믿습니다.</h2>
      <p>좋은 글은 수많은 사람들에게 큰 영향을 미치고, 시간이 지나서 다시 읽어도 그 가치가 오롯이 살아있습니다. KEDITOR는 좋은 글을 쓰고 싶은 모든 이들을 위해 준비한 도구입니다. 한편의 글이 작품이 되는 KEDITOR를 만나보세요.</p>
      <figure data-ke-type="image" data-ke-style="center">
        <img src="https://beta.daumcdn.net/tistory_test/test/c05b06c033b8da5b96e1362b19869c44" width="530" height="329" />
        <figcaption>최적화된 콘텐츠의 다양한 뷰</figcaption>
      </figure>
      <h2>이미지 그리드 레이아웃</h2>
      <p>다양한 이미지 배치를 할 수 있는 그리드 레이아웃 기능</p>
    `;
    
    const rng = editor.selection.getSel().getRangeAt(0);
    rng.setStart(editor.dom.select('p')[0].firstChild, 0);
    rng.setEnd(editor.dom.select('h2')[1].firstChild, 3);
    editor.selection.setRng(rng);

    doc.getElementById('mceu_0-button').click();
    doc.getElementById('mceu_0-button').click();
    doc.getElementsByClassName('colorPalette-preset-td')[0].click();

    LegacyUnit.equal(
      editor.getContent(),
      `<h2>우리는 좋은 글이 가지는 힘을 믿습니다.</h2>\
<p><span style="color: #4ca3f5;">좋은 글은 수많은 사람들에게 큰 영향을 미치고, 시간이 지나서 다시 읽어도 그 가치가 오롯이 살아있습니다. KEDITOR는 좋은 글을 쓰고 싶은 모든 이들을 위해 준비한 도구입니다. 한편의 글이 작품이 되는 KEDITOR를 만나보세요.</span></p>\
<figure data-ke-type="image" data-ke-style="center"><img src="https://beta.daumcdn.net/tistory_test/test/c05b06c033b8da5b96e1362b19869c44" width="530" height="329" />\
<figcaption>최적화된 콘텐츠의 다양한 뷰</figcaption>\
</figure>\
<h2><span style="color: #4ca3f5;">이미지</span> 그리드 레이아웃</h2>\
<p>다양한 이미지 배치를 할 수 있는 그리드 레이아웃 기능</p>`,
    )
  })

  TinyLoader.setup(function (editor, onSuccess, onFailure) {
    Pipeline.async({}, suite.toSteps(editor), onSuccess, onFailure);
  }, {
    indent: false,
    extended_valid_elements: 'b/strong,s/strike,s/del,i/em,iframe[mapdata|src|id|width|height|frameborder|scrolling|data-*]',
    entities: 'raw',
    theme: 'classic',
    convert_fonts_to_spans: false,
    convert_urls: false,
    relative_urls: false,
    forced_root_block: false,
    plugins: 'colorPalette',
    toolbar: 'colorPalette',
    colorPalette: {
      hilite: true,
      module: ['preset', 'palette'], // 'preset', 'palette', 'code'
      color_map_preset: ["#4CA3F5", "#83D25A", "#D83D2E", "#EEBB4C", "#DD6CA4", "#333333", "#214E7A", "#326D22", "#EF9C93", "#EF9842", "#EE95C3", "#929292"],
      color_map_palette: ["#FFD8D8", "#FAE0D4", "#FAECC5", "#FAF4C0", "#E4F7BA", "#CEFBC9", "#D4F4FA", "#D9E5FF", "#DAD9FF", "#E8D9FF", "#FFD9FA", "#FFD9EC", "#F6F6F6", "#EAEAEA", "#FFA7A7", "#FFC19E", "#FFE08C", "#FAED7D", "#CEF279", "#B7F0B1", "#B2EBF4", "#B2CCFF", "#B5B2FF", "#D1B2FF", "#FFB2F5", "#FFB2D9", "#D5D5D5", "#BDBDBD", "#F15F5F", "#F29661", "#F2CB61", "#E5D85C", "#BCE55C", "#86E57F", "#5CD1E5", "#6699FF", "#6B66FF", "#A366FF", "#F261DF", "#F261AA", "#A6A6A6", "#8C8C8C", "#CC3D3D", "#CC723D", "#CCA63D", "#C4B73B", "#9FC93C", "#47C83E", "#3DB7CC", "#4174D9", "#4641D9", "#7E41D9", "#D941C5", "#D9418D", "#747474", "#5D5D5D", "#980000", "#993800", "#997000", "#998A00", "#6B9900", "#2F9D27", "#008299", "#003399", "#050099", "#3D0099", "#990085", "#99004C", "#4C4C4C", "#353535", "#670000", "#662500", "#664B00", "#665C00", "#476600", "#22741C", "#005766", "#002266", "#030066", "#290066", "#660058", "#660033", "#212121", "#000000"],
      preset_rows: 2,
      preset_cols: 6,
      palette_rows: 6,
      palette_cols: 14,
      default_color: '#333333',
    },
    valid_children : 'p[span|b|u|i|s|a|#text|br],-h2[img|div|figure|b],figcaption[#text],figure[img|figcaption|br|a],-span[img]',
    skin_url: 'src/skins/classic'
  }, success, failure);
});
