import { Assertions, Pipeline, UiFinder, GeneralSteps, Logger, Step, Chain, RawAssertions, Waiter, RealMouse } from '@ephox/agar';
import { LegacyUnit, TinyLoader, TinyApis, TinyUi } from '@ephox/mcagar';
import { Element } from '@ephox/sugar';
import { Fun } from '@ephox/katamari';
import Theme from 'src/themes/classic/main/ts/Theme';
import { UnitTest } from '@ephox/bedrock';
import plugin from './../../../src/plugins/newPaste/src/plugin';
// import moreLess from './../../../src/plugins/moreLess/src/plugin';
import Uploader from './../../../src/lib/Uploader';
import * as sinon from 'sinon';
import axios from 'axios';

UnitTest.asynctest('browser.NewPasteTest', function () {
  const success = arguments[arguments.length - 2];
  const failure = arguments[arguments.length - 1];
  const suite = LegacyUnit.createSuite();

  Theme();
  window.tinymce.PluginManager.add('newPaste', plugin);
  // window.tinymce.PluginManager.add('moreLess', moreLess);

  TinyLoader.setup(function (editor, onSuccess, onFailure) {

    const tinyApis = TinyApis(editor);
    const tinyUi = TinyUi(editor);

    const _kImageSettings = editor.settings.kImage;
    const uploader = new Uploader(editor, _kImageSettings.upload_url, _kImageSettings.upload_handler);
    editor.addCommand('KImageUploadUrl', uploader.execUploadForOption)
    // editor.addCommand('uploadTest', () => {
    //   // axios.post('/api/uploadtest', JSON.stringify({"name":"sol"}))
    //   axios.post('/custom/imageUpload')
    //   .then(res => res.data)
    //   .then(result => editor.insertContent(`<h1>${result}</h1>`))
    // })

    const cWaitForState = function (hasState) {
      return function (label, selector) {
        return Chain.fromChainsWith(Element.fromDom(editor.getBody()), [
          UiFinder.cWaitForState(label, selector, hasState)
        ]);
      };
    }

    const cWaitForContent = function(label, selector) {
      return cWaitForState(Fun.constant(true))(label, selector);
    };

    const sWaitForContent = function (label, selector) {
      return Chain.asStep({}, [
        cWaitForContent(label, selector),
      ])
    };

    const cSetTableSelection = function(row, col, length) {
      return Chain.op(function () {
        const tdList = editor.dom.select('td');
        for(let i = 0; i <= row; i++) {
          for(let j = 0; j <= col; j++) {
            if(i <= row && j <= col) {
              tdList[i + j * length].setAttribute('data-mce-selected', true);
            }
          }
        }
      })
    }


    const sSetTableSelection = function (row, col, size) {
      return Chain.asStep({}, [
        cSetTableSelection(row, col, size)
      ])
    }

    const cGetContent = Chain.mapper(function (input) {
      return editor.getContent().replace(/[\s]+/g, ' ')
    });
    
    const sAssertContent = function(expected) {
      return Chain.asStep({}, [
        cGetContent,
        Assertions.cAssertHtml('Checking TinyMCE content', expected)
      ]);
    }

    const assertHtmlExceptId = function(label, expected, actual) {
      const idRegex = /[\s]id=".*?"/;
      const splitExpected = expected.split(idRegex);
      const splitActual = actual.split(idRegex);

      RawAssertions.assertEq('assert html except id', splitExpected[0], splitActual[0]);
      RawAssertions.assertEq('assert html except id', splitExpected[1], splitActual[1]);
    }

    const cAssertHtmlExceptId = function(label, expected) {
      return Chain.op(function (actual) {
        assertHtmlExceptId(label, expected, actual);
      })
    }

    const sAssertHtmlExceptId = function(expected) {
      return Chain.asStep({}, [
        cGetContent,
        cAssertHtmlExceptId('Checking TinyMCE content', expected)
      ]);
    }

    const sExecuteWithMockup = function(input) {
      return Chain.asStep({}, [
        cSetupMockup(input),
        // cExecCommand,
        cRestoreMockup
      ])
    }

    const cSetupMockup = function(response) {
      return Chain.mapper(function (input) {
        const mockServer = sinon.useFakeServer();
        mockServer.respondWith("POST", "*", response);
        return mockServer;
      })
    }

    const cExecCommand = Chain.mapper(function (input) {
      editor.execCommand('makeOpengraph');
      return input;
    })

    const cRestoreMockup = Chain.mapper(function(input: sinon.SinonFakeServer) {
      input.respond();
      input.restore();
    })
    

    Pipeline.async({}, [
      Logger.t('기본적인 html 붙여넣기 테스트', GeneralSteps.sequence([
        tinyApis.sFocus,
        tinyApis.sExecCommand('insertClipboardContent', {
          htmlItem: '<p>Hello world!</p>'
        }),
        sAssertContent(
          '<p>Hello world!</p>'
        ),
      ])),

      Logger.t('색 필터링 테스트(허용된 색)', GeneralSteps.sequence([
        tinyApis.sSetContent(''),
        tinyApis.sFocus,
        tinyApis.sExecCommand('insertClipboardContent', {
          htmlItem: '<p><span style="color: rgb(216, 61, 46);">Hello</span> world!</p>'
        }),
        sAssertContent(
          '<p><span style="color: #d83d2e;">Hello</span> world!</p>'
        ),
      ])),

      Logger.t('색 필터링 테스트(허용되지 않은 색)', GeneralSteps.sequence([
        tinyApis.sSetContent(''),
        tinyApis.sFocus,
        tinyApis.sExecCommand('insertClipboardContent', {
          htmlItem: '<p><span style="color: rgb(214, 61, 46);">Hello</span> world!</p>'
        }),
        sAssertContent(
          '<p>Hello world!</p>'
        ),
      ])),

      Logger.t('붙여넣기 위치 조정 테스트 : 이미지 선택 후 p 붙여넣기', GeneralSteps.sequence([
        tinyApis.sSetContent(`<figure data-ke-type="image" data-ke-style="center"><img src="https://beta.daumcdn.net/tistory_test/test/c05b06c033b8da5b96e1362b19869c44" width="530" height="329"><figcaption>최적화된 콘텐츠의 다양한 뷰</figcaption></figure>`),
        tinyApis.sFocus,
        tinyApis.sSelect('figure', []),
        tinyApis.sExecCommand('insertClipboardContent', {
          htmlItem: '<p>Hello world!</p>'
        }),
        sAssertContent(
          '<figure data-ke-type="image" data-ke-style="center"><img src="https://beta.daumcdn.net/tistory_test/test/c05b06c033b8da5b96e1362b19869c44" width="530" height="329" /><figcaption>최적화된 콘텐츠의 다양한 뷰</figcaption></figure><p>Hello world!</p>'
        ),
      ])),

      Logger.t('figcaption 내에 붙여넣기 테스트', GeneralSteps.sequence([
        tinyApis.sSetContent(`<figure data-ke-type="image" data-ke-style="center"><img src="https://beta.daumcdn.net/tistory_test/test/c05b06c033b8da5b96e1362b19869c44" width="530" height="329"><figcaption>테스트</figcaption></figure>`),
        tinyApis.sFocus,
        tinyApis.sSetSelection([0, 1], 0, [0, 1], 0),
        tinyApis.sExecCommand('insertClipboardContent', {
          htmlItem: '<p>안녕하세요<span style="color: rgb(214, 61, 46);">감사해요</span></p><figure data-ke-type="image" data-ke-style="center"><img src="https://beta.daumcdn.net/tistory_test/test/c05b06c033b8da5b96e1362b19869c44" width="530" height="329" /><figcaption>이 텍스트만 들어가야 한다</figcaption></figure><p>Hello world!</p>'
        }),
        sAssertContent(
          '<figure data-ke-type="image" data-ke-style="center"><img src="https://beta.daumcdn.net/tistory_test/test/c05b06c033b8da5b96e1362b19869c44" width="530" height="329" /><figcaption>테스트안녕하세요감사해요이 텍스트만 들어가야 한다Hello world!</figcaption></figure>'
        ),
      ])),

      Logger.t('url의 형태의 text/plain 링크 삽입 테스트', GeneralSteps.sequence([
        tinyApis.sSetContent(''),
        tinyApis.sFocus,
        tinyApis.sExecCommand('insertClipboardContent', {
          plainTextItem: 'https://www.daum.net'
        }),
        sAssertContent(
          '<p><a href="https://www.daum.net" target="_blank" rel="noopener">https://www.daum.net</a></p>'
        ),
      ])),

      Logger.t('내부 컨텐츠 붙여넣기 : 이미지', GeneralSteps.sequence([
        tinyApis.sSetContent(''),
        tinyApis.sFocus,
        tinyApis.sExecCommand('insertClipboardContent', {
          htmlItem: '<figure data-ke-type="image" data-ke-style="center"><img src="https://beta.daumcdn.net/tistory_test/test/c05b06c033b8da5b96e1362b19869c44" width="530" height="329"><figcaption>최적화된 콘텐츠의 다양한 뷰</figcaption></figure>'
        }),
        sAssertContent(
          '<figure data-ke-type="image" data-ke-style="center"><img src="https://beta.daumcdn.net/tistory_test/test/c05b06c033b8da5b96e1362b19869c44" width="530" height="329" /><figcaption>최적화된 콘텐츠의 다양한 뷰</figcaption></figure><p> </p>'
        ),
      ])),

      Logger.t('내부 컨텐츠 붙여넣기 : 동영상', GeneralSteps.sequence([
        tinyApis.sSetContent(''),
        tinyApis.sFocus,
        tinyApis.sExecCommand('insertClipboardContent', {
          htmlItem: '<figure id="video_1526548034083" contenteditable="false" data-ke-type="video" data-video-vid="v282eL9SWdL2NP5WlxWchIP" data-video-thumbnail="https://t1.daumcdn.net/tvpot/thumb/v282eL9SWdL2NP5WlxWchIP/thumb.png.edit" data-video-url="https://tv.kakao.com/embed/player/cliplink/v282eL9SWdL2NP5WlxWchIP@my?service=daum_tistory" data-video-uploader-host="uploaderx17.videofarm.daum.net" data-mce-selected="ke"><img src="https://t1.daumcdn.net/tvpot/thumb/v282eL9SWdL2NP5WlxWchIP/thumb.png.edit" data-mce-src="https://t1.daumcdn.net/tvpot/thumb/v282eL9SWdL2NP5WlxWchIP/thumb.png.edit"><figcaption contenteditable="true"></figcaption></figure>'
        }),
        sAssertHtmlExceptId(
          '<figure id="video_1526548034083" contenteditable="false" data-ke-type="video" data-video-vid="v282eL9SWdL2NP5WlxWchIP" data-video-thumbnail="https://t1.daumcdn.net/tvpot/thumb/v282eL9SWdL2NP5WlxWchIP/thumb.png.edit" data-video-url="https://tv.kakao.com/embed/player/cliplink/v282eL9SWdL2NP5WlxWchIP@my?service=daum_tistory" data-video-uploader-host="uploaderx17.videofarm.daum.net"><img src="https://t1.daumcdn.net/tvpot/thumb/v282eL9SWdL2NP5WlxWchIP/thumb.png.edit" /><figcaption contenteditable="true"></figcaption></figure><p> </p>'
        ),
      ])),
      
      Logger.t('셀렉션 collapse 아닌 상태에서 붙여넣기 시 삭제 테스트', GeneralSteps.sequence([
        tinyApis.sSetContent(`<p>Icon of <span style="color: rgb(214, 61, 46);">symmetry</span></p><figure data-ke-type="image" data-ke-style="center"><img src="https://beta.daumcdn.net/tistory_test/test/c05b06c033b8da5b96e1362b19869c44" width="530" height="329"><figcaption>최적화된 콘텐츠의 다양한 뷰</figcaption></figure><p>swallowing sides</p>`),
        tinyApis.sFocus,
        tinyApis.sSetSelection([0, 1], 0, [2, 0], 'swallowing'.length),
        tinyApis.sExecCommand('insertClipboardContent', {
          htmlItem: '<p>fall down in front of me</p>'
        }),
        sAssertContent(
          '<p>Icon of fall down in front of me sides</p>'
        ),
      ])),

      Logger.t('더보기 영역 외부->내부 붙여넣기 테스트', GeneralSteps.sequence([
        tinyApis.sSetContent('<div data-ke-type="moreLess" data-text-more="더보기" data-text-less="닫기"><p><br data-mce-bogus="1"></p></div>'),
        tinyApis.sFocus,
        tinyApis.sSetSelection([0, 0], 0, [0, 0], 0),
        tinyApis.sExecCommand('insertClipboardContent', {
          htmlItem: '<p>Hello world!</p><figure data-ke-type="image" data-ke-style="center"><img src="https://beta.daumcdn.net/tistory_test/test/c05b06c033b8da5b96e1362b19869c44" width="530" height="329"><figcaption>최적화된 콘텐츠의 다양한 뷰</figcaption></figure>'
        }),
        sAssertContent(
          '<div data-ke-type="moreLess" data-text-more="더보기" data-text-less="닫기"><p>Hello world!</p><figure data-ke-type="image" data-ke-style="center"><img src="https://beta.daumcdn.net/tistory_test/test/c05b06c033b8da5b96e1362b19869c44" width="530" height="329" /><figcaption>최적화된 콘텐츠의 다양한 뷰</figcaption></figure><p> </p></div>'
        )
      ])),
      
      Logger.t('잘라내기 테스트: 여러 요소', GeneralSteps.sequence([
        tinyApis.sSetContent(`<p>Hello world!</p><figure data-ke-type="image" data-ke-style="center"><img src="https://beta.daumcdn.net/tistory_test/test/c05b06c033b8da5b96e1362b19869c44" width="530" height="329"><figcaption>최적화된 콘텐츠의 다양한 뷰</figcaption></figure><p>this that</p>`),
        tinyApis.sFocus,
        tinyApis.sSetSelection([0, 0], 'Hello'.length, [2, 0], 'This'.length),
        tinyUi.sClickOnMenu('Click Edit menu', 'button:contains("Edit")'),
        tinyUi.sWaitForUi('Wait for dropdown', '.mce-floatpanel[role="application"]'),
        RealMouse.sClickOn('.mce-stack-layout-item:nth-child(4)'),
        sAssertContent('<p>Hello that</p>'),
      ])),

      Logger.t('잘라내기 테스트: figure', GeneralSteps.sequence([
        tinyApis.sSetContent(`<p>Hello world!</p><figure data-ke-type="image" data-ke-style="center"><img src="https://beta.daumcdn.net/tistory_test/test/c05b06c033b8da5b96e1362b19869c44" width="530" height="329"><figcaption>최적화된 콘텐츠의 다양한 뷰</figcaption></figure><p>Hello world!</p>`),
        tinyApis.sFocus,
        tinyApis.sSelect('img', []),
        tinyUi.sClickOnMenu('Click Edit menu', 'button:contains("Edit")'),
        tinyUi.sWaitForUi('Wait for dropdown', '.mce-floatpanel[role="application"]'),
        RealMouse.sClickOn('.mce-stack-layout-item:nth-child(4)'),
        sAssertContent('<p>Hello world!</p><p>Hello world!</p>'),
      ])),

      Logger.t('붙여넣기 테스트: 표', GeneralSteps.sequence([
        tinyApis.sSetContent(''),
        tinyApis.sFocus,
        tinyApis.sExecCommand('insertClipboardContent', {
          htmlItem: '<table><tbody><tr><td>1</td><td>2</td></tr><tr><td>3</td><td>4</td></tr></tbody></table>'
        }),
        sAssertContent('<table style="border-collapse: collapse; width: 100%;" border="1"><tbody><tr><td>1</td><td>2</td></tr><tr><td>3</td><td>4</td></tr></tbody></table>'),
      ])),

      Logger.t('붙여넣기 테스트: 내부 컨텐츠 표', GeneralSteps.sequence([
        tinyApis.sSetContent(''),
        tinyApis.sFocus,
        tinyApis.sExecCommand('insertClipboardContent', {
          htmlItem: '<table style="border-collapse: collapse; width: 100%;" border="1" data-mce-style="border-collapse: collapse; width: 100%;"><tbody><tr><td style="width: 33.3333%;" data-mce-style="width: 33.3333%;" data-mce-selected="1" data-mce-first-selected="1">1</td><td style="width: 33.3333%;" data-mce-style="width: 33.3333%;" data-mce-selected="1">2</td><td style="width: 33.3333%;" data-mce-style="width: 33.3333%;" data-mce-selected="1">3</td></tr><tr><td style="width: 33.3333%;" data-mce-style="width: 33.3333%;" data-mce-selected="1">4</td><td style="width: 33.3333%;" data-mce-style="width: 33.3333%;" data-mce-selected="1">5</td><td style="width: 33.3333%;" data-mce-style="width: 33.3333%;" data-mce-selected="1">6</td></tr><tr><td style="width: 33.3333%;" data-mce-style="width: 33.3333%;" data-mce-selected="1">7</td><td style="width: 33.3333%;" data-mce-style="width: 33.3333%;" data-mce-selected="1">8</td><td style="width: 33.3333%;" data-mce-style="width: 33.3333%;" data-mce-selected="1" data-mce-last-selected="1">9</td></tr></tbody></table>'
        }),
        sAssertContent('<table style="border-collapse: collapse; width: 100%;" border="1"><tbody><tr><td style="width: 33.3333%;">1</td><td style="width: 33.3333%;">2</td><td style="width: 33.3333%;">3</td></tr><tr><td style="width: 33.3333%;">4</td><td style="width: 33.3333%;">5</td><td style="width: 33.3333%;">6</td></tr><tr><td style="width: 33.3333%;">7</td><td style="width: 33.3333%;">8</td><td style="width: 33.3333%;">9</td></tr></tbody></table>'),
      ])),

      Logger.t('잘라내기 테스트: 표 일부', GeneralSteps.sequence([
        tinyApis.sFocus,
        tinyApis.sSetSelection([0,0,1,1], 0, [0,0,1,1], 1),
        sSetTableSelection(1, 1, 3),
        tinyUi.sClickOnMenu('Click Edit menu', 'button:contains("Edit")'),
        tinyUi.sWaitForUi('Wait for dropdown', '.mce-floatpanel[role="application"]'),
        RealMouse.sClickOn('.mce-stack-layout-item:nth-child(4)'),
        sAssertContent('<table style="border-collapse: collapse; width: 100%;" border="1"><tbody><tr><td style="width: 33.3333%;"> </td><td style="width: 33.3333%;"> </td><td style="width: 33.3333%;">3</td></tr><tr><td style="width: 33.3333%;"> </td><td style="width: 33.3333%;"> </td><td style="width: 33.3333%;">6</td></tr><tr><td style="width: 33.3333%;">7</td><td style="width: 33.3333%;">8</td><td style="width: 33.3333%;">9</td></tr></tbody></table>')
      ])),

      Logger.t('잘라내기 테스트: 표 전체', GeneralSteps.sequence([
        tinyApis.sFocus,
        tinyApis.sSetSelection([0,0,2,2], 0, [0,0,2,2], 1),
        sSetTableSelection(2, 2, 3),
        tinyUi.sClickOnMenu('Click Edit menu', 'button:contains("Edit")'),
        tinyUi.sWaitForUi('Wait for dropdown', '.mce-floatpanel[role="application"]'),
        RealMouse.sClickOn('.mce-stack-layout-item:nth-child(4)'),
        sAssertContent('')
      ])),

      // 아래 테스트는 CORS 때문에 불가능(localhost에서 테스트가 동작하기 때문)
      // Logger.t('이미지 붙여넣기 테스트(로컬 파일 아님)', GeneralSteps.sequence([
      //   tinyApis.sSetContent(''),
      //   tinyApis.sFocus,
      //   tinyApis.sExecCommand('insertClipboardContent', {
      //     htmlItem: `<meta charset='utf-8'><img src="https://s.pstatic.net/shopping.phinf/20181018_9/d917a194-620c-4281-92a9-f56a103b4fa2.jpg"/>`
      //   }),
      //   sWaitForContent('wait for content', 'img'),
      //   sExecuteWithMockup(''),
      //   sAssertContent(
      //     '<p>Hello world!</p>'
      //   ),
      // ])),
    ], onSuccess, onFailure);
  }, {
    menubar: true,
    indent: false,
    extended_valid_elements: 'b/strong,s/strike,s/del,i/em,iframe[mapdata|src|id|width|height|frameborder|scrolling|data-*]',
    entities: 'raw',
    theme: 'classic',
    convert_fonts_to_spans: false,
    convert_urls: false,
    relative_urls: false,
    plugins: 'newPaste',
    // toolbar: '',
    kPaste: {
      valid_elements: ['P','IMG', 'H1', 'H2', 'H3', 'TABLE', 'TBODY', 'TR', 'TD', 'B', 'S', 'U', 'A', 'I', 'SPAN', 'UL', 'OL', 'LI', 'HR', 'BLOCKQUOTE'],
      extended_valid_elements: {STRONG: 'b', STRIKE: 's', DEL: 's'},
      invalid_table_children: ['TABLE', 'IMG', 'FIGURE'],
      valid_attributes: ['href', 'src', 'rowspan', 'colspan', 'data-ke-type', 'data-mce-style', 'width', 'height', 'id', 'contenteditable', 'data-video-vid', 'data-video-thumbnail', 'data-video-url', 'data-video-uploader-host', 'data-maps-data', 'data-ke-style'],
    },
    colorPalette: {
      hilite: true,
      module: ['preset', 'palette'], // 'preset', 'palette', 'code'
      color_map_preset: ["#4CA3F5", "#83D25A", "#D83D2E", "#EEBB4C", "#DD6CA4", "#333333", "#214E7A", "#326D22", "#EF9C93", "#EF9842", "#EE95C3", "#929292"],
      color_map_palette: ["#FFD8D8", "#FAE0D4", "#FAECC5", "#FAF4C0", "#E4F7BA", "#CEFBC9", "#D4F4FA", "#D9E5FF", "#DAD9FF", "#E8D9FF", "#FFD9FA", "#FFD9EC", "#F6F6F6", "#EAEAEA", "#FFA7A7", "#FFC19E", "#FFE08C", "#FAED7D", "#CEF279", "#B7F0B1", "#B2EBF4", "#B2CCFF", "#B5B2FF", "#D1B2FF", "#FFB2F5", "#FFB2D9", "#D5D5D5", "#BDBDBD", "#F15F5F", "#F29661", "#F2CB61", "#E5D85C", "#BCE55C", "#86E57F", "#5CD1E5", "#6699FF", "#6B66FF", "#A366FF", "#F261DF", "#F261AA", "#A6A6A6", "#8C8C8C", "#CC3D3D", "#CC723D", "#CCA63D", "#C4B73B", "#9FC93C", "#47C83E", "#3DB7CC", "#4174D9", "#4641D9", "#7E41D9", "#D941C5", "#D9418D", "#747474", "#5D5D5D", "#980000", "#993800", "#997000", "#998A00", "#6B9900", "#2F9D27", "#008299", "#003399", "#050099", "#3D0099", "#990085", "#99004C", "#4C4C4C", "#353535", "#670000", "#662500", "#664B00", "#665C00", "#476600", "#22741C", "#005766", "#002266", "#030066", "#290066", "#660058", "#660033", "#212121", "#000000"],
      preset_rows: 2,
      preset_cols: 6,
      palette_rows: 6,
      palette_cols: 14,
      default_color: '#333333',
    },
    kImage: {
      upload_url: "/api/upload",
      is_image_align: true,
      is_image_edit: true,
      is_caption: true,
      is_grid_layout: true,
      is_representative_image: true,
      representative_image_callback: function(data) {
        console.log('대표 이미지 데이터 전달', data)
      }
    },
    valid_children : 'p[span|b|u|i|s|a|#text|br],-h2[img|div|figure|b],figcaption[#text],figure[img|figcaption|br|a],-span[img]',
    skin_url: 'src/skins/classic'
  }, success, failure);
});
