import { Chain, Pipeline, Step, UiFinder, Assertions, GeneralSteps, Logger } from '@ephox/agar';
import { UnitTest } from '@ephox/bedrock';
import { TinyApis, TinyLoader, TinyUi } from '@ephox/mcagar';
import { Element } from '@ephox/sugar';
import { Fun } from '@ephox/katamari';
import axios from 'axios';
import Theme from 'src/themes/classic/main/ts/Theme';
import plugin from '../../../src/plugins/newOg/src/plugin';
import * as sinon from 'sinon';

UnitTest.asynctest('browser.OpengraphTest', function () {
  const success = arguments[arguments.length - 2];
  const failure = arguments[arguments.length - 1];

  Theme();
  window.tinymce.PluginManager.add('newOg', plugin);

  TinyLoader.setup(function (editor, onSuccess, onFailure) {
    
    const tinyApis = TinyApis(editor);
    const ui = TinyUi(editor);

    const cWaitForState = function (hasState) {
      return function (label, selector) {
        return Chain.fromChainsWith(Element.fromDom(editor.getBody()), [
          UiFinder.cWaitForState(label, selector, hasState)
        ]);
      };
    }

    const cWaitForContent = function(label, selector) {
      return cWaitForState(Fun.constant(true))(label, selector);
    };

    const sWaitForContent = function (label, selector) {
      return Chain.asStep({}, [
        cWaitForContent(label, selector),
      ])
    };

    const cGetContent = Chain.mapper(function (input) {
      return editor.getContent().replace(/[\s]+/g, ' ')
    });

    const sAssertContent = function(expected) {
      return Chain.asStep({}, [
        cGetContent,
        Assertions.cAssertHtml('Checking TinyMCE content', expected)
      ]);
    }

    const sExecuteWithMockup = function(input) {
      return Chain.asStep({}, [
        cSetupMockup(input),
        cExecCommand,
        cRestoreMockup
      ])
    }

    const cSetupMockup = function(response) {
      return Chain.mapper(function (input) {
        const mockServer = sinon.useFakeServer();
        mockServer.respondWith("POST", "http://goscrap.kakao.com/scrap/v6", response);
        return mockServer;
      })
    }

    const cExecCommand = Chain.mapper(function (input) {
      editor.execCommand('makeOpengraph');
      return input;
    })

    const cRestoreMockup = Chain.mapper(function(input: sinon.SinonFakeServer) {
      input.respond();
      input.restore();
    })
    
    Pipeline.async({}, [
      Logger.t('다음 테스트', GeneralSteps.sequence([
        tinyApis.sFocus,
        tinyApis.sSetContent('<p>https://www.daum.net</p>'),
        tinyApis.sExecCommand('selectAll'),
        tinyApis.sExecCommand('mceInsertLink', 'https://www.daum.net'),
        sExecuteWithMockup(`{"code":0,"description":"Mockup Daum","host":"www.daum.net","image":["http://t1.beta.daumcdn.net/keditor/test/ff761b281df881516788c8f9f207cdbc"],"is_opengraph":true,"meta":{"cached":true},"requested_url":"https://www.daum.net","section":"","title":"Daum","type":"website","url":"https://www.daum.net/"}`),
        sWaitForContent('wait for ui', '.og-image'),
        Step.sync(() => {
          // id는 timestamp 기반이기 때문에 임의로 변경
          editor.dom.select('figure')[0].id = 'og_1234567890';
        }),
        sAssertContent(
          '<p><a href="https://www.daum.net">https://www.daum.net</a></p><figure id="og_1234567890" contenteditable="false" data-ke-type="opengraph"><a href="https://www.daum.net/"><div class="og-text"><p class="og-title">Daum</p><p class="og-desc">Mockup Daum</p><p class="og-host">www.daum.net</p></div><div class="og-image" style="background-image: url(\'http://t1.beta.daumcdn.net/keditor/test/ff761b281df881516788c8f9f207cdbc\'); background-color: #333;"> </div></a></figure><p> </p>'
        ),
      ])),

      Logger.t('네이버 테스트', GeneralSteps.sequence([
        tinyApis.sFocus,
        tinyApis.sSetContent('<p>https://www.naver.com</p>'),
        tinyApis.sExecCommand('selectAll'),
        tinyApis.sExecCommand('mceInsertLink', 'https://www.naver.com'),
        sExecuteWithMockup(`{"code":0,"description":"Mockup Naver","host":"www.naver.com","image":["http://t1.beta.daumcdn.net/keditor/test/ff761b281df881516788c8f9f207cdbc"],"is_opengraph":true,"meta":{"cached":true},"requested_url":"https://www.naver.com","section":"","title":"Naver","type":"website","url":"https://www.naver.com/"}`),
        sWaitForContent('wait for ui', '.og-image'),
        Step.sync(() => {
          editor.dom.select('figure')[0].id = 'og_1234567891';
        }),
        sAssertContent(
          '<p><a href="https://www.naver.com">https://www.naver.com</a></p><figure id="og_1234567891" contenteditable="false" data-ke-type="opengraph"><a href="https://www.naver.com/"><div class="og-text"><p class="og-title">Naver</p><p class="og-desc">Mockup Naver</p><p class="og-host">www.naver.com</p></div><div class="og-image" style="background-image: url(\'http://t1.beta.daumcdn.net/keditor/test/ff761b281df881516788c8f9f207cdbc\'); background-color: #333;"> </div></a></figure><p> </p>'
        ),
      ])),

      Logger.t('유튜브 테스트', GeneralSteps.sequence([
        tinyApis.sFocus,
        tinyApis.sSetContent('<p>https://www.youtube.com/watch?v=jNQXAC9IVRw</p>'),
        tinyApis.sExecCommand('selectAll'),
        tinyApis.sExecCommand('mceInsertLink', 'https://www.youtube.com/watch?v=jNQXAC9IVRw'),
        sExecuteWithMockup(`{"code":0,"description":"The first video on YouTube. Maybe it's time to go back to the zoo? The name of the music playing in the background is Darude - Sandstorm.","host":"www.youtube.com","image":["http://t1.beta.daumcdn.net/keditor/test/ff761b281df881516788c8f9f207cdbc"],"is_opengraph":true,"meta":{"cached":true},"requested_url":"https://www.youtube.com/watch?v=jNQXAC9IVRw","section":"","site_name":"YouTube","title":"Me at the zoo","type":"video.other","url":"https://www.youtube.com/watch?v=jNQXAC9IVRw"}`),
        sWaitForContent('wait for ui', 'span'),
        Step.sync(() => {
          editor.dom.select('figure')[0].id = 'og_1234567892';
        }),
        sAssertContent(
          '<p><a href="https://www.youtube.com/watch?v=jNQXAC9IVRw">https://www.youtube.com/watch?v=jNQXAC9IVRw</a></p><figure id="og_1234567892" contenteditable="false" data-ke-type="opengraph-video-youtube" data-video-url="https://www.youtube.com/watch?v=jNQXAC9IVRw" data-img-src="http://t1.beta.daumcdn.net/keditor/test/ff761b281df881516788c8f9f207cdbc"><img src="http://t1.beta.daumcdn.net/keditor/test/ff761b281df881516788c8f9f207cdbc" /><div> </div></figure><p> </p>'
        ),
      ])),

      Logger.t('비메오 테스트', GeneralSteps.sequence([
        tinyApis.sFocus,
        tinyApis.sSetContent('<p>https://vimeo.com/52553715</p>'),
        tinyApis.sExecCommand('selectAll'),
        tinyApis.sExecCommand('mceInsertLink', 'https://vimeo.com/52553715'),
        sExecuteWithMockup(`{"code":0,"description":"My best flights in 2012, flying trough the crazy scenery of the alps while enjoying my faboulous EnZo. Edit by Jan Sterren www.airaddicts.ch flights: Fiesch…","host":"vimeo.com","image":["http://t1.beta.daumcdn.net/keditor/test/ff761b281df881516788c8f9f207cdbc"],"is_opengraph":true,"meta":{"cached":false},"requested_url":"https://vimeo.com/52553715","section":"","site_name":"Vimeo","title":"paragliding | best of xc-flying 2012","type":"video","url":"https://vimeo.com/52553715"}`),
        sWaitForContent('wait for ui', 'span'),
        Step.sync(() => {
          editor.dom.select('figure')[0].id = 'og_1234567893';
        }),
        sAssertContent(
          '<p><a href="https://vimeo.com/52553715">https://vimeo.com/52553715</a></p><figure id="og_1234567893" contenteditable="false" data-ke-type="opengraph-video-vimeo" data-video-url="https://vimeo.com/52553715" data-img-src="http://t1.beta.daumcdn.net/keditor/test/ff761b281df881516788c8f9f207cdbc"><img src="http://t1.beta.daumcdn.net/keditor/test/ff761b281df881516788c8f9f207cdbc" /><div> </div></figure><p> </p>'
        ),
      ])),

      Logger.t('카카오tv 테스트', GeneralSteps.sequence([
        tinyApis.sFocus,
        tinyApis.sSetContent('<p>http://tv.kakao.com/v/391668447</p>'),
        tinyApis.sExecCommand('selectAll'),
        tinyApis.sExecCommand('mceInsertLink', 'http://tv.kakao.com/v/391668447'),
        sExecuteWithMockup(`{"code":0,"description":"My best flights in 2012, flying trough the crazy scenery of the alps while enjoying my faboulous EnZo. Edit by Jan Sterren www.airaddicts.ch flights: Fiesch…","host":"vimeo.com","image":["http://t1.beta.daumcdn.net/keditor/test/ff761b281df881516788c8f9f207cdbc"],"is_opengraph":true,"meta":{"cached":false},"requested_url":"http://tv.kakao.com/v/391668447","section":"","site_name":"Vimeo","title":"paragliding | best of xc-flying 2012","type":"video","url":"http://tv.kakao.com/v/391668447"}`),
        sWaitForContent('wait for ui', 'span'),
        Step.sync(() => {
          editor.dom.select('figure')[0].id = 'og_1234567893';
        }),
        sAssertContent(
          '<p><a href="http://tv.kakao.com/v/391668447">http://tv.kakao.com/v/391668447</a></p><figure id="og_1234567893" contenteditable="false" data-ke-type="opengraph-video-vimeo" data-video-url="http://tv.kakao.com/v/391668447" data-img-src="http://t1.beta.daumcdn.net/keditor/test/ff761b281df881516788c8f9f207cdbc"><img src="http://t1.beta.daumcdn.net/keditor/test/ff761b281df881516788c8f9f207cdbc" /><div> </div></figure><p> </p>'
        ),
      ])),
    ], onSuccess, onFailure);

  }, {
    indent: false,
    valid_children : 'p[span|b|u|i|s|a|#text|br],-h2[img|div|figure|b],figcaption[#text],figure[img|figcaption|br|a|div|span|p],-span[img],+a[div],-li[blockquote|h2|h3]',
    extended_valid_elements: 'b/strong,s/strike,s/del,i/em,iframe[mapdata|src|id|width|height|frameborder|scrolling|data-*]',
    entities: 'raw',
    theme: 'classic',
    convert_fonts_to_spans: false,
    convert_urls: false,
    relative_urls: false,
    forced_root_block: false,
    plugins: 'newOg',
    toolbar: '',
    skin_url: 'src/skins/classic',
    opengraph: {
      fetch_handler: function(url, callback) {
        const payload = new FormData();
        payload.append('url', url);
        payload.append('origin', 'keditor');
        axios.post('http://goscrap.kakao.com/scrap/v6', payload)
          .then(function(res) { return res.data })
          .then(function(data) { callback(data) })
      }
    },
  }, success, failure);
});